import time

COUNTRY_MAP = {
    "FR": {"lang": "fr-FR", "region": "FR", "corsDomain": "fr.finance.yahoo.com"},
    "IN": {"lang": "en-IN", "region": "IN", "corsDomain": "in.finance.yahoo.com"},
    "HK": {"lang": "zh-Hant-HK", "region": "HK", "corsDomain": "hk.finance.yahoo.com"},
    "DE": {"lang": "de-DE", "region": "DE", "corsDomain": "de.finance.yahoo.com"},
    "CA": {"lang": "en-CA", "region": "CA", "corsDomain": "ca.finance.yahoo.com"},
    "ES": {"lang": "es-ES", "region": "ES", "corsDomain": "es.finance.yahoo.com"},
    "IT": {"lang": "it-IT", "region": "IT", "corsDomain": "it.finance.yahoo.com"},
    "US": {"lang": "en-US", "region": "US", "corsDomain": "finance.yahoo.com"},
    "AU": {"lang": "en-AU", "region": "AU", "corsDomain": "au.finance.yahoo.com"},
    "UK": {"lang": "en-GB", "region": "GB", "corsDomain": "uk.finance.yahoo.com"},
    "BR": {"lang": "pt-BR", "region": "BR", "corsDomain": "br.financas.yahoo.com"},
    "NZ": {"lang": "en-NZ", "region": "NZ", "corsDomain": "nz.finance.yahoo.com"},
    "SG": {"lang": "en-SG", "region": "SG", "corsDomain": "sg.finance.yahoo.com"},
    "TW": {"lang": "zh-tw", "region": "TW", "corsDomain": "tw.finance.yahoo.com"},
}

FUNDAMENTALS_MAP = {
    "income_statement": {
        "annual": [
            "annualAmortization",
            "annualAmortizationOfIntangiblesIncomeStatement",
            "annualAverageDilutionEarnings",
            "annualBasicAccountingChange",
            "annualBasicAverageShares",
            "annualBasicContinuousOperations",
            "annualBasicDiscontinuousOperations",
            "annualBasicEPS",
            "annualBasicEPSOtherGainsLosses",
            "annualBasicExtraordinary",
            "annualContinuingAndDiscontinuedBasicEPS",
            "annualContinuingAndDiscontinuedDilutedEPS",
            "annualCostOfRevenue",
            "annualDepletionIncomeStatement",
            "annualDepreciationAmortizationDepletionIncomeStatement",
            "annualDepreciationAndAmortizationInIncomeStatement",
            "annualDepreciationIncomeStatement",
            "annualDilutedAccountingChange",
            "annualDilutedAverageShares",
            "annualDilutedContinuousOperations",
            "annualDilutedDiscontinuousOperations",
            "annualDilutedEPS",
            "annualDilutedEPSOtherGainsLosses",
            "annualDilutedExtraordinary",
            "annualDilutedNIAvailtoComStockholders",
            "annualDividendPerShare",
            "annualEBIT",
            "annualEBITDA",
            "annualEarningsFromEquityInterest",
            "annualEarningsFromEquityInterestNetOfTax",
            "annualExciseTaxes",
            "annualGainOnSaleOfBusiness",
            "annualGainOnSaleOfPPE",
            "annualGainOnSaleOfSecurity",
            "annualGeneralAndAdministrativeExpense",
            "annualGrossProfit",
            "annualImpairmentOfCapitalAssets",
            "annualInsuranceAndClaims",
            "annualInterestExpense",
            "annualInterestExpenseNonOperating",
            "annualInterestIncome",
            "annualInterestIncomeNonOperating",
            "annualMinorityInterests",
            "annualNetIncome",
            "annualNetIncomeCommonStockholders",
            "annualNetIncomeContinuousOperations",
            "annualNetIncomeDiscontinuousOperations",
            "annualNetIncomeExtraordinary",
            "annualNetIncomeFromContinuingAndDiscontinuedOperation",
            "annualNetIncomeFromContinuingOperationNetMinorityInterest",
            "annualNetIncomeFromTaxLossCarryforward",
            "annualNetIncomeIncludingNoncontrollingInterests",
            "annualNetInterestIncome",
            "annualNetNonOperatingInterestIncomeExpense",
            "annualNormalizedBasicEPS",
            "annualNormalizedDilutedEPS",
            "annualNormalizedEBITDA",
            "annualNormalizedIncome",
            "annualOperatingExpense",
            "annualOperatingIncome",
            "annualOperatingRevenue",
            "annualOtherGandA",
            "annualOtherIncomeExpense",
            "annualOtherNonOperatingIncomeExpenses",
            "annualOtherOperatingExpenses",
            "annualOtherSpecialCharges",
            "annualOtherTaxes",
            "annualOtherunderPreferredStockDividend",
            "annualPreferredStockDividends",
            "annualPretaxIncome",
            "annualProvisionForDoubtfulAccounts",
            "annualReconciledCostOfRevenue",
            "annualReconciledDepreciation",
            "annualRentAndLandingFees",
            "annualRentExpenseSupplemental",
            "annualReportedNormalizedBasicEPS",
            "annualReportedNormalizedDilutedEPS",
            "annualResearchAndDevelopment",
            "annualRestructuringAndMergernAcquisition",
            "annualSalariesAndWages",
            "annualSecuritiesAmortization",
            "annualSellingAndMarketingExpense",
            "annualSellingGeneralAndAdministration",
            "annualSpecialIncomeCharges",
            "annualTaxEffectOfUnusualItems",
            "annualTaxLossCarryforwardBasicEPS",
            "annualTaxLossCarryforwardDilutedEPS",
            "annualTaxProvision",
            "annualTaxRateForCalcs",
            "annualTotalExpenses",
            "annualTotalOperatingIncomeAsReported",
            "annualTotalOtherFinanceCost",
            "annualTotalRevenue",
            "annualTotalUnusualItems",
            "annualTotalUnusualItemsExcludingGoodwill",
            "annualWriteOff",
            "trailingAmortization",
            "trailingAmortizationOfIntangiblesIncomeStatement",
            "trailingAverageDilutionEarnings",
            "trailingBasicAccountingChange",
            "trailingBasicAverageShares",
            "trailingBasicContinuousOperations",
            "trailingBasicDiscontinuousOperations",
            "trailingBasicEPS",
            "trailingBasicEPSOtherGainsLosses",
            "trailingBasicExtraordinary",
            "trailingContinuingAndDiscontinuedBasicEPS",
            "trailingContinuingAndDiscontinuedDilutedEPS",
            "trailingCostOfRevenue",
            "trailingDepletionIncomeStatement",
            "trailingDepreciationAmortizationDepletionIncomeStatement",
            "trailingDepreciationAndAmortizationInIncomeStatement",
            "trailingDepreciationIncomeStatement",
            "trailingDilutedAccountingChange",
            "trailingDilutedAverageShares",
            "trailingDilutedContinuousOperations",
            "trailingDilutedDiscontinuousOperations",
            "trailingDilutedEPS",
            "trailingDilutedEPSOtherGainsLosses",
            "trailingDilutedExtraordinary",
            "trailingDilutedNIAvailtoComStockholders",
            "trailingDividendPerShare",
            "trailingEBIT",
            "trailingEBITDA",
            "trailingEarningsFromEquityInterest",
            "trailingEarningsFromEquityInterestNetOfTax",
            "trailingExciseTaxes",
            "trailingGainOnSaleOfBusiness",
            "trailingGainOnSaleOfPPE",
            "trailingGainOnSaleOfSecurity",
            "trailingGeneralAndAdministrativeExpense",
            "trailingGrossProfit",
            "trailingImpairmentOfCapitalAssets",
            "trailingInsuranceAndClaims",
            "trailingInterestExpense",
            "trailingInterestExpenseNonOperating",
            "trailingInterestIncome",
            "trailingInterestIncomeNonOperating",
            "trailingMinorityInterests",
            "trailingNetIncome",
            "trailingNetIncomeCommonStockholders",
            "trailingNetIncomeContinuousOperations",
            "trailingNetIncomeDiscontinuousOperations",
            "trailingNetIncomeExtraordinary",
            "trailingNetIncomeFromContinuingAndDiscontinuedOperation",
            "trailingNetIncomeFromContinuingOperationNetMinorityInterest",
            "trailingNetIncomeFromTaxLossCarryforward",
            "trailingNetIncomeIncludingNoncontrollingInterests",
            "trailingNetInterestIncome",
            "trailingNetNonOperatingInterestIncomeExpense",
            "trailingNormalizedBasicEPS",
            "trailingNormalizedDilutedEPS",
            "trailingNormalizedEBITDA",
            "trailingNormalizedIncome",
            "trailingOperatingExpense",
            "trailingOperatingIncome",
            "trailingOperatingRevenue",
            "trailingOtherGandA",
            "trailingOtherIncomeExpense",
            "trailingOtherNonOperatingIncomeExpenses",
            "trailingOtherOperatingExpenses",
            "trailingOtherSpecialCharges",
            "trailingOtherTaxes",
            "trailingOtherunderPreferredStockDividend",
            "trailingPreferredStockDividends",
            "trailingPretaxIncome",
            "trailingProvisionForDoubtfulAccounts",
            "trailingReconciledCostOfRevenue",
            "trailingReconciledDepreciation",
            "trailingRentAndLandingFees",
            "trailingRentExpenseSupplemental",
            "trailingReportedNormalizedBasicEPS",
            "trailingReportedNormalizedDilutedEPS",
            "trailingResearchAndDevelopment",
            "trailingRestructuringAndMergernAcquisition",
            "trailingSalariesAndWages",
            "trailingSecuritiesAmortization",
            "trailingSellingAndMarketingExpense",
            "trailingSellingGeneralAndAdministration",
            "trailingSpecialIncomeCharges",
            "trailingTaxEffectOfUnusualItems",
            "trailingTaxLossCarryforwardBasicEPS",
            "trailingTaxLossCarryforwardDilutedEPS",
            "trailingTaxProvision",
            "trailingTaxRateForCalcs",
            "trailingTotalExpenses",
            "trailingTotalOperatingIncomeAsReported",
            "trailingTotalOtherFinanceCost",
            "trailingTotalRevenue",
            "trailingTotalUnusualItems",
            "trailingTotalUnusualItemsExcludingGoodwill",
            "trailingWriteOff",
        ],
        "quarterly": [
            "quarterlyAmortization",
            "quarterlyAmortizationOfIntangiblesIncomeStatement",
            "quarterlyAverageDilutionEarnings",
            "quarterlyBasicAccountingChange",
            "quarterlyBasicAverageShares",
            "quarterlyBasicContinuousOperations",
            "quarterlyBasicDiscontinuousOperations",
            "quarterlyBasicEPS",
            "quarterlyBasicEPSOtherGainsLosses",
            "quarterlyBasicExtraordinary",
            "quarterlyContinuingAndDiscontinuedBasicEPS",
            "quarterlyContinuingAndDiscontinuedDilutedEPS",
            "quarterlyCostOfRevenue",
            "quarterlyDepletionIncomeStatement",
            "quarterlyDepreciationAmortizationDepletionIncomeStatement",
            "quarterlyDepreciationAndAmortizationInIncomeStatement",
            "quarterlyDepreciationIncomeStatement",
            "quarterlyDilutedAccountingChange",
            "quarterlyDilutedAverageShares",
            "quarterlyDilutedContinuousOperations",
            "quarterlyDilutedDiscontinuousOperations",
            "quarterlyDilutedEPS",
            "quarterlyDilutedEPSOtherGainsLosses",
            "quarterlyDilutedExtraordinary",
            "quarterlyDilutedNIAvailtoComStockholders",
            "quarterlyDividendPerShare",
            "quarterlyEBIT",
            "quarterlyEBITDA",
            "quarterlyEarningsFromEquityInterest",
            "quarterlyEarningsFromEquityInterestNetOfTax",
            "quarterlyExciseTaxes",
            "quarterlyGainOnSaleOfBusiness",
            "quarterlyGainOnSaleOfPPE",
            "quarterlyGainOnSaleOfSecurity",
            "quarterlyGeneralAndAdministrativeExpense",
            "quarterlyGrossProfit",
            "quarterlyImpairmentOfCapitalAssets",
            "quarterlyInsuranceAndClaims",
            "quarterlyInterestExpense",
            "quarterlyInterestExpenseNonOperating",
            "quarterlyInterestIncome",
            "quarterlyInterestIncomeNonOperating",
            "quarterlyMinorityInterests",
            "quarterlyNetIncome",
            "quarterlyNetIncomeCommonStockholders",
            "quarterlyNetIncomeContinuousOperations",
            "quarterlyNetIncomeDiscontinuousOperations",
            "quarterlyNetIncomeExtraordinary",
            "quarterlyNetIncomeFromContinuingAndDiscontinuedOperation",
            "quarterlyNetIncomeFromContinuingOperationNetMinorityInterest",
            "quarterlyNetIncomeFromTaxLossCarryforward",
            "quarterlyNetIncomeIncludingNoncontrollingInterests",
            "quarterlyNetInterestIncome",
            "quarterlyNetNonOperatingInterestIncomeExpense",
            "quarterlyNormalizedBasicEPS",
            "quarterlyNormalizedDilutedEPS",
            "quarterlyNormalizedEBITDA",
            "quarterlyNormalizedIncome",
            "quarterlyOperatingExpense",
            "quarterlyOperatingIncome",
            "quarterlyOperatingRevenue",
            "quarterlyOtherGandA",
            "quarterlyOtherIncomeExpense",
            "quarterlyOtherNonOperatingIncomeExpenses",
            "quarterlyOtherOperatingExpenses",
            "quarterlyOtherSpecialCharges",
            "quarterlyOtherTaxes",
            "quarterlyOtherunderPreferredStockDividend",
            "quarterlyPreferredStockDividends",
            "quarterlyPretaxIncome",
            "quarterlyProvisionForDoubtfulAccounts",
            "quarterlyReconciledCostOfRevenue",
            "quarterlyReconciledDepreciation",
            "quarterlyRentAndLandingFees",
            "quarterlyRentExpenseSupplemental",
            "quarterlyReportedNormalizedBasicEPS",
            "quarterlyReportedNormalizedDilutedEPS",
            "quarterlyResearchAndDevelopment",
            "quarterlyRestructuringAndMergernAcquisition",
            "quarterlySalariesAndWages",
            "quarterlySecuritiesAmortization",
            "quarterlySellingAndMarketingExpense",
            "quarterlySellingGeneralAndAdministration",
            "quarterlySpecialIncomeCharges",
            "quarterlyTaxEffectOfUnusualItems",
            "quarterlyTaxLossCarryforwardBasicEPS",
            "quarterlyTaxLossCarryforwardDilutedEPS",
            "quarterlyTaxProvision",
            "quarterlyTaxRateForCalcs",
            "quarterlyTotalExpenses",
            "quarterlyTotalOperatingIncomeAsReported",
            "quarterlyTotalOtherFinanceCost",
            "quarterlyTotalRevenue",
            "quarterlyTotalUnusualItems",
            "quarterlyTotalUnusualItemsExcludingGoodwill",
            "quarterlyWriteOff",
            "trailingAmortization",
            "trailingAmortizationOfIntangiblesIncomeStatement",
            "trailingAverageDilutionEarnings",
            "trailingBasicAccountingChange",
            "trailingBasicAverageShares",
            "trailingBasicContinuousOperations",
            "trailingBasicDiscontinuousOperations",
            "trailingBasicEPS",
            "trailingBasicEPSOtherGainsLosses",
            "trailingBasicExtraordinary",
            "trailingContinuingAndDiscontinuedBasicEPS",
            "trailingContinuingAndDiscontinuedDilutedEPS",
            "trailingCostOfRevenue",
            "trailingDepletionIncomeStatement",
            "trailingDepreciationAmortizationDepletionIncomeStatement",
            "trailingDepreciationAndAmortizationInIncomeStatement",
            "trailingDepreciationIncomeStatement",
            "trailingDilutedAccountingChange",
            "trailingDilutedAverageShares",
            "trailingDilutedContinuousOperations",
            "trailingDilutedDiscontinuousOperations",
            "trailingDilutedEPS",
            "trailingDilutedEPSOtherGainsLosses",
            "trailingDilutedExtraordinary",
            "trailingDilutedNIAvailtoComStockholders",
            "trailingDividendPerShare",
            "trailingEBIT",
            "trailingEBITDA",
            "trailingEarningsFromEquityInterest",
            "trailingEarningsFromEquityInterestNetOfTax",
            "trailingExciseTaxes",
            "trailingGainOnSaleOfBusiness",
            "trailingGainOnSaleOfPPE",
            "trailingGainOnSaleOfSecurity",
            "trailingGeneralAndAdministrativeExpense",
            "trailingGrossProfit",
            "trailingImpairmentOfCapitalAssets",
            "trailingInsuranceAndClaims",
            "trailingInterestExpense",
            "trailingInterestExpenseNonOperating",
            "trailingInterestIncome",
            "trailingInterestIncomeNonOperating",
            "trailingMinorityInterests",
            "trailingNetIncome",
            "trailingNetIncomeCommonStockholders",
            "trailingNetIncomeContinuousOperations",
            "trailingNetIncomeDiscontinuousOperations",
            "trailingNetIncomeExtraordinary",
            "trailingNetIncomeFromContinuingAndDiscontinuedOperation",
            "trailingNetIncomeFromContinuingOperationNetMinorityInterest",
            "trailingNetIncomeFromTaxLossCarryforward",
            "trailingNetIncomeIncludingNoncontrollingInterests",
            "trailingNetInterestIncome",
            "trailingNetNonOperatingInterestIncomeExpense",
            "trailingNormalizedBasicEPS",
            "trailingNormalizedDilutedEPS",
            "trailingNormalizedEBITDA",
            "trailingNormalizedIncome",
            "trailingOperatingExpense",
            "trailingOperatingIncome",
            "trailingOperatingRevenue",
            "trailingOtherGandA",
            "trailingOtherIncomeExpense",
            "trailingOtherNonOperatingIncomeExpenses",
            "trailingOtherOperatingExpenses",
            "trailingOtherSpecialCharges",
            "trailingOtherTaxes",
            "trailingOtherunderPreferredStockDividend",
            "trailingPreferredStockDividends",
            "trailingPretaxIncome",
            "trailingProvisionForDoubtfulAccounts",
            "trailingReconciledCostOfRevenue",
            "trailingReconciledDepreciation",
            "trailingRentAndLandingFees",
            "trailingRentExpenseSupplemental",
            "trailingReportedNormalizedBasicEPS",
            "trailingReportedNormalizedDilutedEPS",
            "trailingResearchAndDevelopment",
            "trailingRestructuringAndMergernAcquisition",
            "trailingSalariesAndWages",
            "trailingSecuritiesAmortization",
            "trailingSellingAndMarketingExpense",
            "trailingSellingGeneralAndAdministration",
            "trailingSpecialIncomeCharges",
            "trailingTaxEffectOfUnusualItems",
            "trailingTaxLossCarryforwardBasicEPS",
            "trailingTaxLossCarryforwardDilutedEPS",
            "trailingTaxProvision",
            "trailingTaxRateForCalcs",
            "trailingTotalExpenses",
            "trailingTotalOperatingIncomeAsReported",
            "trailingTotalOtherFinanceCost",
            "trailingTotalRevenue",
            "trailingTotalUnusualItems",
            "trailingTotalUnusualItemsExcludingGoodwill",
            "trailingWriteOff"
        ],
        "monthly": [
            "monthlyAmortization",
            "monthlyAmortizationOfIntangiblesIncomeStatement",
            "monthlyAverageDilutionEarnings",
            "monthlyBasicAccountingChange",
            "monthlyBasicAverageShares",
            "monthlyBasicContinuousOperations",
            "monthlyBasicDiscontinuousOperations",
            "monthlyBasicEPS",
            "monthlyBasicEPSOtherGainsLosses",
            "monthlyBasicExtraordinary",
            "monthlyContinuingAndDiscontinuedBasicEPS",
            "monthlyContinuingAndDiscontinuedDilutedEPS",
            "monthlyCostOfRevenue",
            "monthlyDepletionIncomeStatement",
            "monthlyDepreciationAmortizationDepletionIncomeStatement",
            "monthlyDepreciationAndAmortizationInIncomeStatement",
            "monthlyDepreciationIncomeStatement",
            "monthlyDilutedAccountingChange",
            "monthlyDilutedAverageShares",
            "monthlyDilutedContinuousOperations",
            "monthlyDilutedDiscontinuousOperations",
            "monthlyDilutedEPS",
            "monthlyDilutedEPSOtherGainsLosses",
            "monthlyDilutedExtraordinary",
            "monthlyDilutedNIAvailtoComStockholders",
            "monthlyDividendPerShare",
            "monthlyEBIT",
            "monthlyEBITDA",
            "monthlyEarningsFromEquityInterest",
            "monthlyEarningsFromEquityInterestNetOfTax",
            "monthlyExciseTaxes",
            "monthlyGainOnSaleOfBusiness",
            "monthlyGainOnSaleOfPPE",
            "monthlyGainOnSaleOfSecurity",
            "monthlyGeneralAndAdministrativeExpense",
            "monthlyGrossProfit",
            "monthlyImpairmentOfCapitalAssets",
            "monthlyInsuranceAndClaims",
            "monthlyInterestExpense",
            "monthlyInterestExpenseNonOperating",
            "monthlyInterestIncome",
            "monthlyInterestIncomeNonOperating",
            "monthlyMinorityInterests",
            "monthlyNetIncome",
            "monthlyNetIncomeCommonStockholders",
            "monthlyNetIncomeContinuousOperations",
            "monthlyNetIncomeDiscontinuousOperations",
            "monthlyNetIncomeExtraordinary",
            "monthlyNetIncomeFromContinuingAndDiscontinuedOperation",
            "monthlyNetIncomeFromContinuingOperationNetMinorityInterest",
            "monthlyNetIncomeFromTaxLossCarryforward",
            "monthlyNetIncomeIncludingNoncontrollingInterests",
            "monthlyNetInterestIncome",
            "monthlyNetNonOperatingInterestIncomeExpense",
            "monthlyNormalizedBasicEPS",
            "monthlyNormalizedDilutedEPS",
            "monthlyNormalizedEBITDA",
            "monthlyNormalizedIncome",
            "monthlyOperatingExpense",
            "monthlyOperatingIncome",
            "monthlyOperatingRevenue",
            "monthlyOtherGandA",
            "monthlyOtherIncomeExpense",
            "monthlyOtherNonOperatingIncomeExpenses",
            "monthlyOtherOperatingExpenses",
            "monthlyOtherSpecialCharges",
            "monthlyOtherTaxes",
            "monthlyOtherunderPreferredStockDividend",
            "monthlyPreferredStockDividends",
            "monthlyPretaxIncome",
            "monthlyProvisionForDoubtfulAccounts",
            "monthlyReconciledCostOfRevenue",
            "monthlyReconciledDepreciation",
            "monthlyRentAndLandingFees",
            "monthlyRentExpenseSupplemental",
            "monthlyReportedNormalizedBasicEPS",
            "monthlyReportedNormalizedDilutedEPS",
            "monthlyResearchAndDevelopment",
            "monthlyRestructuringAndMergernAcquisition",
            "monthlySalariesAndWages",
            "monthlySecuritiesAmortization",
            "monthlySellingAndMarketingExpense",
            "monthlySellingGeneralAndAdministration",
            "monthlySpecialIncomeCharges",
            "monthlyTaxEffectOfUnusualItems",
            "monthlyTaxLossCarryforwardBasicEPS",
            "monthlyTaxLossCarryforwardDilutedEPS",
            "monthlyTaxProvision",
            "monthlyTaxRateForCalcs",
            "monthlyTotalExpenses",
            "monthlyTotalOperatingIncomeAsReported",
            "monthlyTotalOtherFinanceCost",
            "monthlyTotalRevenue",
            "monthlyTotalUnusualItems",
            "monthlyTotalUnusualItemsExcludingGoodwill",
            "monthlyWriteOff",
            "trailingAmortization",
            "trailingAmortizationOfIntangiblesIncomeStatement",
            "trailingAverageDilutionEarnings",
            "trailingBasicAccountingChange",
            "trailingBasicAverageShares",
            "trailingBasicContinuousOperations",
            "trailingBasicDiscontinuousOperations",
            "trailingBasicEPS",
            "trailingBasicEPSOtherGainsLosses",
            "trailingBasicExtraordinary",
            "trailingContinuingAndDiscontinuedBasicEPS",
            "trailingContinuingAndDiscontinuedDilutedEPS",
            "trailingCostOfRevenue",
            "trailingDepletionIncomeStatement",
            "trailingDepreciationAmortizationDepletionIncomeStatement",
            "trailingDepreciationAndAmortizationInIncomeStatement",
            "trailingDepreciationIncomeStatement",
            "trailingDilutedAccountingChange",
            "trailingDilutedAverageShares",
            "trailingDilutedContinuousOperations",
            "trailingDilutedDiscontinuousOperations",
            "trailingDilutedEPS",
            "trailingDilutedEPSOtherGainsLosses",
            "trailingDilutedExtraordinary",
            "trailingDilutedNIAvailtoComStockholders",
            "trailingDividendPerShare",
            "trailingEBIT",
            "trailingEBITDA",
            "trailingEarningsFromEquityInterest",
            "trailingEarningsFromEquityInterestNetOfTax",
            "trailingExciseTaxes",
            "trailingGainOnSaleOfBusiness",
            "trailingGainOnSaleOfPPE",
            "trailingGainOnSaleOfSecurity",
            "trailingGeneralAndAdministrativeExpense",
            "trailingGrossProfit",
            "trailingImpairmentOfCapitalAssets",
            "trailingInsuranceAndClaims",
            "trailingInterestExpense",
            "trailingInterestExpenseNonOperating",
            "trailingInterestIncome",
            "trailingInterestIncomeNonOperating",
            "trailingMinorityInterests",
            "trailingNetIncome",
            "trailingNetIncomeCommonStockholders",
            "trailingNetIncomeContinuousOperations",
            "trailingNetIncomeDiscontinuousOperations",
            "trailingNetIncomeExtraordinary",
            "trailingNetIncomeFromContinuingAndDiscontinuedOperation",
            "trailingNetIncomeFromContinuingOperationNetMinorityInterest",
            "trailingNetIncomeFromTaxLossCarryforward",
            "trailingNetIncomeIncludingNoncontrollingInterests",
            "trailingNetInterestIncome",
            "trailingNetNonOperatingInterestIncomeExpense",
            "trailingNormalizedBasicEPS",
            "trailingNormalizedDilutedEPS",
            "trailingNormalizedEBITDA",
            "trailingNormalizedIncome",
            "trailingOperatingExpense",
            "trailingOperatingIncome",
            "trailingOperatingRevenue",
            "trailingOtherGandA",
            "trailingOtherIncomeExpense",
            "trailingOtherNonOperatingIncomeExpenses",
            "trailingOtherOperatingExpenses",
            "trailingOtherSpecialCharges",
            "trailingOtherTaxes",
            "trailingOtherunderPreferredStockDividend",
            "trailingPreferredStockDividends",
            "trailingPretaxIncome",
            "trailingProvisionForDoubtfulAccounts",
            "trailingReconciledCostOfRevenue",
            "trailingReconciledDepreciation",
            "trailingRentAndLandingFees",
            "trailingRentExpenseSupplemental",
            "trailingReportedNormalizedBasicEPS",
            "trailingReportedNormalizedDilutedEPS",
            "trailingResearchAndDevelopment",
            "trailingRestructuringAndMergernAcquisition",
            "trailingSalariesAndWages",
            "trailingSecuritiesAmortization",
            "trailingSellingAndMarketingExpense",
            "trailingSellingGeneralAndAdministration",
            "trailingSpecialIncomeCharges",
            "trailingTaxEffectOfUnusualItems",
            "trailingTaxLossCarryforwardBasicEPS",
            "trailingTaxLossCarryforwardDilutedEPS",
            "trailingTaxProvision",
            "trailingTaxRateForCalcs",
            "trailingTotalExpenses",
            "trailingTotalOperatingIncomeAsReported",
            "trailingTotalOtherFinanceCost",
            "trailingTotalRevenue",
            "trailingTotalUnusualItems",
            "trailingTotalUnusualItemsExcludingGoodwill",
            "trailingWriteOff",
        ]
    },
    "balance_sheet": {
        "annual": [
            "annualAccountsPayable",
            "annualAccountsReceivable",
            "annualAccruedInterestReceivable",
            "annualAccumulatedDepreciation",
            "annualAdditionalPaidInCapital",
            "annualAllowanceForDoubtfulAccountsReceivable",
            "annualAssetsHeldForSaleCurrent",
            "annualAvailableForSaleSecurities",
            "annualBuildingsAndImprovements",
            "annualCapitalLeaseObligations",
            "annualCapitalStock",
            "annualCashAndCashEquivalents",
            "annualCashCashEquivalentsAndShortTermInvestments",
            "annualCashEquivalents",
            "annualCashFinancial",
            "annualCommercialPaper",
            "annualCommonStock",
            "annualCommonStockEquity",
            "annualConstructionInProgress",
            "annualCurrentAccruedExpenses",
            "annualCurrentAssets",
            "annualCurrentCapitalLeaseObligation",
            "annualCurrentDebt",
            "annualCurrentDebtAndCapitalLeaseObligation",
            "annualCurrentDeferredAssets",
            "annualCurrentDeferredLiabilities",
            "annualCurrentDeferredRevenue",
            "annualCurrentDeferredTaxesAssets",
            "annualCurrentDeferredTaxesLiabilities",
            "annualCurrentLiabilities",
            "annualCurrentNotesPayable",
            "annualCurrentProvisions",
            "annualDefinedPensionBenefit",
            "annualDerivativeProductLiabilities",
            "annualDividendsPayable",
            "annualDuefromRelatedPartiesCurrent",
            "annualDuefromRelatedPartiesNonCurrent",
            "annualDuetoRelatedPartiesCurrent",
            "annualDuetoRelatedPartiesNonCurrent",
            "annualEmployeeBenefits",
            "annualFinancialAssets",
            "annualFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
            "annualFinishedGoods",
            "annualFixedAssetsRevaluationReserve",
            "annualForeignCurrencyTranslationAdjustments",
            "annualGainsLossesNotAffectingRetainedEarnings",
            "annualGeneralPartnershipCapital",
            "annualGoodwill",
            "annualGoodwillAndOtherIntangibleAssets",
            "annualGrossAccountsReceivable",
            "annualGrossPPE",
            "annualHedgingAssetsCurrent",
            "annualHeldToMaturitySecurities",
            "annualIncomeTaxPayable",
            "annualInterestPayable",
            "annualInventoriesAdjustmentsAllowances",
            "annualInventory",
            "annualInvestedCapital",
            "annualInvestmentProperties",
            "annualInvestmentinFinancialAssets",
            "annualInvestmentsAndAdvances",
            "annualInvestmentsInOtherVenturesUnderEquityMethod",
            "annualInvestmentsinAssociatesatCost",
            "annualInvestmentsinJointVenturesatCost",
            "annualInvestmentsinSubsidiariesatCost",
            "annualLandAndImprovements",
            "annualLeases",
            "annualLiabilitiesHeldforSaleNonCurrent",
            "annualLimitedPartnershipCapital",
            "annualLineOfCredit",
            "annualLoansReceivable",
            "annualLongTermCapitalLeaseObligation",
            "annualLongTermDebt",
            "annualLongTermDebtAndCapitalLeaseObligation",
            "annualLongTermEquityInvestment",
            "annualLongTermProvisions",
            "annualMachineryFurnitureEquipment",
            "annualMinimumPensionLiabilities",
            "annualMinorityInterest",
            "annualNetDebt",
            "annualNetPPE",
            "annualNetTangibleAssets",
            "annualNonCurrentAccountsReceivable",
            "annualNonCurrentAccruedExpenses",
            "annualNonCurrentDeferredAssets",
            "annualNonCurrentDeferredLiabilities",
            "annualNonCurrentDeferredRevenue",
            "annualNonCurrentDeferredTaxesAssets",
            "annualNonCurrentDeferredTaxesLiabilities",
            "annualNonCurrentNoteReceivables",
            "annualNonCurrentPensionAndOtherPostretirementBenefitPlans",
            "annualNonCurrentPrepaidAssets",
            "annualNotesReceivable",
            "annualOrdinarySharesNumber",
            "annualOtherCapitalStock",
            "annualOtherCurrentAssets",
            "annualOtherCurrentBorrowings",
            "annualOtherCurrentLiabilities",
            "annualOtherEquityAdjustments",
            "annualOtherEquityInterest",
            "annualOtherIntangibleAssets",
            "annualOtherInventories",
            "annualOtherInvestments",
            "annualOtherNonCurrentAssets",
            "annualOtherNonCurrentLiabilities",
            "annualOtherPayable",
            "annualOtherProperties",
            "annualOtherReceivables",
            "annualOtherShortTermInvestments",
            "annualPayables",
            "annualPayablesAndAccruedExpenses",
            "annualPensionandOtherPostRetirementBenefitPlansCurrent",
            "annualPreferredSecuritiesOutsideStockEquity",
            "annualPreferredSharesNumber",
            "annualPreferredStock",
            "annualPreferredStockEquity",
            "annualPrepaidAssets",
            "annualProperties",
            "annualRawMaterials",
            "annualReceivables",
            "annualReceivablesAdjustmentsAllowances",
            "annualRestrictedCash",
            "annualRestrictedCommonStock",
            "annualRetainedEarnings",
            "annualShareIssued",
            "annualStockholdersEquity",
            "annualTangibleBookValue",
            "annualTaxesReceivable",
            "annualTotalAssets",
            "annualTotalCapitalization",
            "annualTotalDebt",
            "annualTotalEquityGrossMinorityInterest",
            "annualTotalLiabilitiesNetMinorityInterest",
            "annualTotalNonCurrentAssets",
            "annualTotalNonCurrentLiabilitiesNetMinorityInterest",
            "annualTotalPartnershipCapital",
            "annualTotalTaxPayable",
            "annualTradeandOtherPayablesNonCurrent",
            "annualTradingSecurities",
            "annualTreasurySharesNumber",
            "annualTreasuryStock",
            "annualUnrealizedGainLoss",
            "annualWorkInProcess",
            "annualWorkingCapital",
            "trailingAccountsPayable",
            "trailingAccountsReceivable",
            "trailingAccruedInterestReceivable",
            "trailingAccumulatedDepreciation",
            "trailingAdditionalPaidInCapital",
            "trailingAllowanceForDoubtfulAccountsReceivable",
            "trailingAssetsHeldForSaleCurrent",
            "trailingAvailableForSaleSecurities",
            "trailingBuildingsAndImprovements",
            "trailingCapitalLeaseObligations",
            "trailingCapitalStock",
            "trailingCashAndCashEquivalents",
            "trailingCashCashEquivalentsAndShortTermInvestments",
            "trailingCashEquivalents",
            "trailingCashFinancial",
            "trailingCommercialPaper",
            "trailingCommonStock",
            "trailingCommonStockEquity",
            "trailingConstructionInProgress",
            "trailingCurrentAccruedExpenses",
            "trailingCurrentAssets",
            "trailingCurrentCapitalLeaseObligation",
            "trailingCurrentDebt",
            "trailingCurrentDebtAndCapitalLeaseObligation",
            "trailingCurrentDeferredAssets",
            "trailingCurrentDeferredLiabilities",
            "trailingCurrentDeferredRevenue",
            "trailingCurrentDeferredTaxesAssets",
            "trailingCurrentDeferredTaxesLiabilities",
            "trailingCurrentLiabilities",
            "trailingCurrentNotesPayable",
            "trailingCurrentProvisions",
            "trailingDefinedPensionBenefit",
            "trailingDerivativeProductLiabilities",
            "trailingDividendsPayable",
            "trailingDuefromRelatedPartiesCurrent",
            "trailingDuefromRelatedPartiesNonCurrent",
            "trailingDuetoRelatedPartiesCurrent",
            "trailingDuetoRelatedPartiesNonCurrent",
            "trailingEmployeeBenefits",
            "trailingFinancialAssets",
            "trailingFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
            "trailingFinishedGoods",
            "trailingFixedAssetsRevaluationReserve",
            "trailingForeignCurrencyTranslationAdjustments",
            "trailingGainsLossesNotAffectingRetainedEarnings",
            "trailingGeneralPartnershipCapital",
            "trailingGoodwill",
            "trailingGoodwillAndOtherIntangibleAssets",
            "trailingGrossAccountsReceivable",
            "trailingGrossPPE",
            "trailingHedgingAssetsCurrent",
            "trailingHeldToMaturitySecurities",
            "trailingIncomeTaxPayable",
            "trailingInterestPayable",
            "trailingInventoriesAdjustmentsAllowances",
            "trailingInventory",
            "trailingInvestedCapital",
            "trailingInvestmentProperties",
            "trailingInvestmentinFinancialAssets",
            "trailingInvestmentsAndAdvances",
            "trailingInvestmentsInOtherVenturesUnderEquityMethod",
            "trailingInvestmentsinAssociatesatCost",
            "trailingInvestmentsinJointVenturesatCost",
            "trailingInvestmentsinSubsidiariesatCost",
            "trailingLandAndImprovements",
            "trailingLeases",
            "trailingLiabilitiesHeldforSaleNonCurrent",
            "trailingLimitedPartnershipCapital",
            "trailingLineOfCredit",
            "trailingLoansReceivable",
            "trailingLongTermCapitalLeaseObligation",
            "trailingLongTermDebt",
            "trailingLongTermDebtAndCapitalLeaseObligation",
            "trailingLongTermEquityInvestment",
            "trailingLongTermProvisions",
            "trailingMachineryFurnitureEquipment",
            "trailingMinimumPensionLiabilities",
            "trailingMinorityInterest",
            "trailingNetDebt",
            "trailingNetPPE",
            "trailingNetTangibleAssets",
            "trailingNonCurrentAccountsReceivable",
            "trailingNonCurrentAccruedExpenses",
            "trailingNonCurrentDeferredAssets",
            "trailingNonCurrentDeferredLiabilities",
            "trailingNonCurrentDeferredRevenue",
            "trailingNonCurrentDeferredTaxesAssets",
            "trailingNonCurrentDeferredTaxesLiabilities",
            "trailingNonCurrentNoteReceivables",
            "trailingNonCurrentPensionAndOtherPostretirementBenefitPlans",
            "trailingNonCurrentPrepaidAssets",
            "trailingNotesReceivable",
            "trailingOrdinarySharesNumber",
            "trailingOtherCapitalStock",
            "trailingOtherCurrentAssets",
            "trailingOtherCurrentBorrowings",
            "trailingOtherCurrentLiabilities",
            "trailingOtherEquityAdjustments",
            "trailingOtherEquityInterest",
            "trailingOtherIntangibleAssets",
            "trailingOtherInventories",
            "trailingOtherInvestments",
            "trailingOtherNonCurrentAssets",
            "trailingOtherNonCurrentLiabilities",
            "trailingOtherPayable",
            "trailingOtherProperties",
            "trailingOtherReceivables",
            "trailingOtherShortTermInvestments",
            "trailingPayables",
            "trailingPayablesAndAccruedExpenses",
            "trailingPensionandOtherPostRetirementBenefitPlansCurrent",
            "trailingPreferredSecuritiesOutsideStockEquity",
            "trailingPreferredSharesNumber",
            "trailingPreferredStock",
            "trailingPreferredStockEquity",
            "trailingPrepaidAssets",
            "trailingProperties",
            "trailingRawMaterials",
            "trailingReceivables",
            "trailingReceivablesAdjustmentsAllowances",
            "trailingRestrictedCash",
            "trailingRestrictedCommonStock",
            "trailingRetainedEarnings",
            "trailingShareIssued",
            "trailingStockholdersEquity",
            "trailingTangibleBookValue",
            "trailingTaxesReceivable",
            "trailingTotalAssets",
            "trailingTotalCapitalization",
            "trailingTotalDebt",
            "trailingTotalEquityGrossMinorityInterest",
            "trailingTotalLiabilitiesNetMinorityInterest",
            "trailingTotalNonCurrentAssets",
            "trailingTotalNonCurrentLiabilitiesNetMinorityInterest",
            "trailingTotalPartnershipCapital",
            "trailingTotalTaxPayable",
            "trailingTradeandOtherPayablesNonCurrent",
            "trailingTradingSecurities",
            "trailingTreasurySharesNumber",
            "trailingTreasuryStock",
            "trailingUnrealizedGainLoss",
            "trailingWorkInProcess",
            "trailingWorkingCapital",
        ],
        "quarterly": [
            "quarterlyAccountsPayable",
            "quarterlyAccountsReceivable",
            "quarterlyAccruedInterestReceivable",
            "quarterlyAccumulatedDepreciation",
            "quarterlyAdditionalPaidInCapital",
            "quarterlyAllowanceForDoubtfulAccountsReceivable",
            "quarterlyAssetsHeldForSaleCurrent",
            "quarterlyAvailableForSaleSecurities",
            "quarterlyBuildingsAndImprovements",
            "quarterlyCapitalLeaseObligations",
            "quarterlyCapitalStock",
            "quarterlyCashAndCashEquivalents",
            "quarterlyCashCashEquivalentsAndShortTermInvestments",
            "quarterlyCashEquivalents",
            "quarterlyCashFinancial",
            "quarterlyCommercialPaper",
            "quarterlyCommonStock",
            "quarterlyCommonStockEquity",
            "quarterlyConstructionInProgress",
            "quarterlyCurrentAccruedExpenses",
            "quarterlyCurrentAssets",
            "quarterlyCurrentCapitalLeaseObligation",
            "quarterlyCurrentDebt",
            "quarterlyCurrentDebtAndCapitalLeaseObligation",
            "quarterlyCurrentDeferredAssets",
            "quarterlyCurrentDeferredLiabilities",
            "quarterlyCurrentDeferredRevenue",
            "quarterlyCurrentDeferredTaxesAssets",
            "quarterlyCurrentDeferredTaxesLiabilities",
            "quarterlyCurrentLiabilities",
            "quarterlyCurrentNotesPayable",
            "quarterlyCurrentProvisions",
            "quarterlyDefinedPensionBenefit",
            "quarterlyDerivativeProductLiabilities",
            "quarterlyDividendsPayable",
            "quarterlyDuefromRelatedPartiesCurrent",
            "quarterlyDuefromRelatedPartiesNonCurrent",
            "quarterlyDuetoRelatedPartiesCurrent",
            "quarterlyDuetoRelatedPartiesNonCurrent",
            "quarterlyEmployeeBenefits",
            "quarterlyFinancialAssets",
            "quarterlyFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
            "quarterlyFinishedGoods",
            "quarterlyFixedAssetsRevaluationReserve",
            "quarterlyForeignCurrencyTranslationAdjustments",
            "quarterlyGainsLossesNotAffectingRetainedEarnings",
            "quarterlyGeneralPartnershipCapital",
            "quarterlyGoodwill",
            "quarterlyGoodwillAndOtherIntangibleAssets",
            "quarterlyGrossAccountsReceivable",
            "quarterlyGrossPPE",
            "quarterlyHedgingAssetsCurrent",
            "quarterlyHeldToMaturitySecurities",
            "quarterlyIncomeTaxPayable",
            "quarterlyInterestPayable",
            "quarterlyInventoriesAdjustmentsAllowances",
            "quarterlyInventory",
            "quarterlyInvestedCapital",
            "quarterlyInvestmentProperties",
            "quarterlyInvestmentinFinancialAssets",
            "quarterlyInvestmentsAndAdvances",
            "quarterlyInvestmentsInOtherVenturesUnderEquityMethod",
            "quarterlyInvestmentsinAssociatesatCost",
            "quarterlyInvestmentsinJointVenturesatCost",
            "quarterlyInvestmentsinSubsidiariesatCost",
            "quarterlyLandAndImprovements",
            "quarterlyLeases",
            "quarterlyLiabilitiesHeldforSaleNonCurrent",
            "quarterlyLimitedPartnershipCapital",
            "quarterlyLineOfCredit",
            "quarterlyLoansReceivable",
            "quarterlyLongTermCapitalLeaseObligation",
            "quarterlyLongTermDebt",
            "quarterlyLongTermDebtAndCapitalLeaseObligation",
            "quarterlyLongTermEquityInvestment",
            "quarterlyLongTermProvisions",
            "quarterlyMachineryFurnitureEquipment",
            "quarterlyMinimumPensionLiabilities",
            "quarterlyMinorityInterest",
            "quarterlyNetDebt",
            "quarterlyNetPPE",
            "quarterlyNetTangibleAssets",
            "quarterlyNonCurrentAccountsReceivable",
            "quarterlyNonCurrentAccruedExpenses",
            "quarterlyNonCurrentDeferredAssets",
            "quarterlyNonCurrentDeferredLiabilities",
            "quarterlyNonCurrentDeferredRevenue",
            "quarterlyNonCurrentDeferredTaxesAssets",
            "quarterlyNonCurrentDeferredTaxesLiabilities",
            "quarterlyNonCurrentNoteReceivables",
            "quarterlyNonCurrentPensionAndOtherPostretirementBenefitPlans",
            "quarterlyNonCurrentPrepaidAssets",
            "quarterlyNotesReceivable",
            "quarterlyOrdinarySharesNumber",
            "quarterlyOtherCapitalStock",
            "quarterlyOtherCurrentAssets",
            "quarterlyOtherCurrentBorrowings",
            "quarterlyOtherCurrentLiabilities",
            "quarterlyOtherEquityAdjustments",
            "quarterlyOtherEquityInterest",
            "quarterlyOtherIntangibleAssets",
            "quarterlyOtherInventories",
            "quarterlyOtherInvestments",
            "quarterlyOtherNonCurrentAssets",
            "quarterlyOtherNonCurrentLiabilities",
            "quarterlyOtherPayable",
            "quarterlyOtherProperties",
            "quarterlyOtherReceivables",
            "quarterlyOtherShortTermInvestments",
            "quarterlyPayables",
            "quarterlyPayablesAndAccruedExpenses",
            "quarterlyPensionandOtherPostRetirementBenefitPlansCurrent",
            "quarterlyPreferredSecuritiesOutsideStockEquity",
            "quarterlyPreferredSharesNumber",
            "quarterlyPreferredStock",
            "quarterlyPreferredStockEquity",
            "quarterlyPrepaidAssets",
            "quarterlyProperties",
            "quarterlyRawMaterials",
            "quarterlyReceivables",
            "quarterlyReceivablesAdjustmentsAllowances",
            "quarterlyRestrictedCash",
            "quarterlyRestrictedCommonStock",
            "quarterlyRetainedEarnings",
            "quarterlyShareIssued",
            "quarterlyStockholdersEquity",
            "quarterlyTangibleBookValue",
            "quarterlyTaxesReceivable",
            "quarterlyTotalAssets",
            "quarterlyTotalCapitalization",
            "quarterlyTotalDebt",
            "quarterlyTotalEquityGrossMinorityInterest",
            "quarterlyTotalLiabilitiesNetMinorityInterest",
            "quarterlyTotalNonCurrentAssets",
            "quarterlyTotalNonCurrentLiabilitiesNetMinorityInterest",
            "quarterlyTotalPartnershipCapital",
            "quarterlyTotalTaxPayable",
            "quarterlyTradeandOtherPayablesNonCurrent",
            "quarterlyTradingSecurities",
            "quarterlyTreasurySharesNumber",
            "quarterlyTreasuryStock",
            "quarterlyUnrealizedGainLoss",
            "quarterlyWorkInProcess",
            "quarterlyWorkingCapital",
            "trailingAccountsPayable",
            "trailingAccountsReceivable",
            "trailingAccruedInterestReceivable",
            "trailingAccumulatedDepreciation",
            "trailingAdditionalPaidInCapital",
            "trailingAllowanceForDoubtfulAccountsReceivable",
            "trailingAssetsHeldForSaleCurrent",
            "trailingAvailableForSaleSecurities",
            "trailingBuildingsAndImprovements",
            "trailingCapitalLeaseObligations",
            "trailingCapitalStock",
            "trailingCashAndCashEquivalents",
            "trailingCashCashEquivalentsAndShortTermInvestments",
            "trailingCashEquivalents",
            "trailingCashFinancial",
            "trailingCommercialPaper",
            "trailingCommonStock",
            "trailingCommonStockEquity",
            "trailingConstructionInProgress",
            "trailingCurrentAccruedExpenses",
            "trailingCurrentAssets",
            "trailingCurrentCapitalLeaseObligation",
            "trailingCurrentDebt",
            "trailingCurrentDebtAndCapitalLeaseObligation",
            "trailingCurrentDeferredAssets",
            "trailingCurrentDeferredLiabilities",
            "trailingCurrentDeferredRevenue",
            "trailingCurrentDeferredTaxesAssets",
            "trailingCurrentDeferredTaxesLiabilities",
            "trailingCurrentLiabilities",
            "trailingCurrentNotesPayable",
            "trailingCurrentProvisions",
            "trailingDefinedPensionBenefit",
            "trailingDerivativeProductLiabilities",
            "trailingDividendsPayable",
            "trailingDuefromRelatedPartiesCurrent",
            "trailingDuefromRelatedPartiesNonCurrent",
            "trailingDuetoRelatedPartiesCurrent",
            "trailingDuetoRelatedPartiesNonCurrent",
            "trailingEmployeeBenefits",
            "trailingFinancialAssets",
            "trailingFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
            "trailingFinishedGoods",
            "trailingFixedAssetsRevaluationReserve",
            "trailingForeignCurrencyTranslationAdjustments",
            "trailingGainsLossesNotAffectingRetainedEarnings",
            "trailingGeneralPartnershipCapital",
            "trailingGoodwill",
            "trailingGoodwillAndOtherIntangibleAssets",
            "trailingGrossAccountsReceivable",
            "trailingGrossPPE",
            "trailingHedgingAssetsCurrent",
            "trailingHeldToMaturitySecurities",
            "trailingIncomeTaxPayable",
            "trailingInterestPayable",
            "trailingInventoriesAdjustmentsAllowances",
            "trailingInventory",
            "trailingInvestedCapital",
            "trailingInvestmentProperties",
            "trailingInvestmentinFinancialAssets",
            "trailingInvestmentsAndAdvances",
            "trailingInvestmentsInOtherVenturesUnderEquityMethod",
            "trailingInvestmentsinAssociatesatCost",
            "trailingInvestmentsinJointVenturesatCost",
            "trailingInvestmentsinSubsidiariesatCost",
            "trailingLandAndImprovements",
            "trailingLeases",
            "trailingLiabilitiesHeldforSaleNonCurrent",
            "trailingLimitedPartnershipCapital",
            "trailingLineOfCredit",
            "trailingLoansReceivable",
            "trailingLongTermCapitalLeaseObligation",
            "trailingLongTermDebt",
            "trailingLongTermDebtAndCapitalLeaseObligation",
            "trailingLongTermEquityInvestment",
            "trailingLongTermProvisions",
            "trailingMachineryFurnitureEquipment",
            "trailingMinimumPensionLiabilities",
            "trailingMinorityInterest",
            "trailingNetDebt",
            "trailingNetPPE",
            "trailingNetTangibleAssets",
            "trailingNonCurrentAccountsReceivable",
            "trailingNonCurrentAccruedExpenses",
            "trailingNonCurrentDeferredAssets",
            "trailingNonCurrentDeferredLiabilities",
            "trailingNonCurrentDeferredRevenue",
            "trailingNonCurrentDeferredTaxesAssets",
            "trailingNonCurrentDeferredTaxesLiabilities",
            "trailingNonCurrentNoteReceivables",
            "trailingNonCurrentPensionAndOtherPostretirementBenefitPlans",
            "trailingNonCurrentPrepaidAssets",
            "trailingNotesReceivable",
            "trailingOrdinarySharesNumber",
            "trailingOtherCapitalStock",
            "trailingOtherCurrentAssets",
            "trailingOtherCurrentBorrowings",
            "trailingOtherCurrentLiabilities",
            "trailingOtherEquityAdjustments",
            "trailingOtherEquityInterest",
            "trailingOtherIntangibleAssets",
            "trailingOtherInventories",
            "trailingOtherInvestments",
            "trailingOtherNonCurrentAssets",
            "trailingOtherNonCurrentLiabilities",
            "trailingOtherPayable",
            "trailingOtherProperties",
            "trailingOtherReceivables",
            "trailingOtherShortTermInvestments",
            "trailingPayables",
            "trailingPayablesAndAccruedExpenses",
            "trailingPensionandOtherPostRetirementBenefitPlansCurrent",
            "trailingPreferredSecuritiesOutsideStockEquity",
            "trailingPreferredSharesNumber",
            "trailingPreferredStock",
            "trailingPreferredStockEquity",
            "trailingPrepaidAssets",
            "trailingProperties",
            "trailingRawMaterials",
            "trailingReceivables",
            "trailingReceivablesAdjustmentsAllowances",
            "trailingRestrictedCash",
            "trailingRestrictedCommonStock",
            "trailingRetainedEarnings",
            "trailingShareIssued",
            "trailingStockholdersEquity",
            "trailingTangibleBookValue",
            "trailingTaxesReceivable",
            "trailingTotalAssets",
            "trailingTotalCapitalization",
            "trailingTotalDebt",
            "trailingTotalEquityGrossMinorityInterest",
            "trailingTotalLiabilitiesNetMinorityInterest",
            "trailingTotalNonCurrentAssets",
            "trailingTotalNonCurrentLiabilitiesNetMinorityInterest",
            "trailingTotalPartnershipCapital",
            "trailingTotalTaxPayable",
            "trailingTradeandOtherPayablesNonCurrent",
            "trailingTradingSecurities",
            "trailingTreasurySharesNumber",
            "trailingTreasuryStock",
            "trailingUnrealizedGainLoss",
            "trailingWorkInProcess",
            "trailingWorkingCapital",
        ],
        "monthly": [
            "monthlyAccountsPayable",
            "monthlyAccountsReceivable",
            "monthlyAccruedInterestReceivable",
            "monthlyAccumulatedDepreciation",
            "monthlyAdditionalPaidInCapital",
            "monthlyAllowanceForDoubtfulAccountsReceivable",
            "monthlyAssetsHeldForSaleCurrent",
            "monthlyAvailableForSaleSecurities",
            "monthlyBuildingsAndImprovements",
            "monthlyCapitalLeaseObligations",
            "monthlyCapitalStock",
            "monthlyCashAndCashEquivalents",
            "monthlyCashCashEquivalentsAndShortTermInvestments",
            "monthlyCashEquivalents",
            "monthlyCashFinancial",
            "monthlyCommercialPaper",
            "monthlyCommonStock",
            "monthlyCommonStockEquity",
            "monthlyConstructionInProgress",
            "monthlyCurrentAccruedExpenses",
            "monthlyCurrentAssets",
            "monthlyCurrentCapitalLeaseObligation",
            "monthlyCurrentDebt",
            "monthlyCurrentDebtAndCapitalLeaseObligation",
            "monthlyCurrentDeferredAssets",
            "monthlyCurrentDeferredLiabilities",
            "monthlyCurrentDeferredRevenue",
            "monthlyCurrentDeferredTaxesAssets",
            "monthlyCurrentDeferredTaxesLiabilities",
            "monthlyCurrentLiabilities",
            "monthlyCurrentNotesPayable",
            "monthlyCurrentProvisions",
            "monthlyDefinedPensionBenefit",
            "monthlyDerivativeProductLiabilities",
            "monthlyDividendsPayable",
            "monthlyDuefromRelatedPartiesCurrent",
            "monthlyDuefromRelatedPartiesNonCurrent",
            "monthlyDuetoRelatedPartiesCurrent",
            "monthlyDuetoRelatedPartiesNonCurrent",
            "monthlyEmployeeBenefits",
            "monthlyFinancialAssets",
            "monthlyFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
            "monthlyFinishedGoods",
            "monthlyFixedAssetsRevaluationReserve",
            "monthlyForeignCurrencyTranslationAdjustments",
            "monthlyGainsLossesNotAffectingRetainedEarnings",
            "monthlyGeneralPartnershipCapital",
            "monthlyGoodwill",
            "monthlyGoodwillAndOtherIntangibleAssets",
            "monthlyGrossAccountsReceivable",
            "monthlyGrossPPE",
            "monthlyHedgingAssetsCurrent",
            "monthlyHeldToMaturitySecurities",
            "monthlyIncomeTaxPayable",
            "monthlyInterestPayable",
            "monthlyInventoriesAdjustmentsAllowances",
            "monthlyInventory",
            "monthlyInvestedCapital",
            "monthlyInvestmentProperties",
            "monthlyInvestmentinFinancialAssets",
            "monthlyInvestmentsAndAdvances",
            "monthlyInvestmentsInOtherVenturesUnderEquityMethod",
            "monthlyInvestmentsinAssociatesatCost",
            "monthlyInvestmentsinJointVenturesatCost",
            "monthlyInvestmentsinSubsidiariesatCost",
            "monthlyLandAndImprovements",
            "monthlyLeases",
            "monthlyLiabilitiesHeldforSaleNonCurrent",
            "monthlyLimitedPartnershipCapital",
            "monthlyLineOfCredit",
            "monthlyLoansReceivable",
            "monthlyLongTermCapitalLeaseObligation",
            "monthlyLongTermDebt",
            "monthlyLongTermDebtAndCapitalLeaseObligation",
            "monthlyLongTermEquityInvestment",
            "monthlyLongTermProvisions",
            "monthlyMachineryFurnitureEquipment",
            "monthlyMinimumPensionLiabilities",
            "monthlyMinorityInterest",
            "monthlyNetDebt",
            "monthlyNetPPE",
            "monthlyNetTangibleAssets",
            "monthlyNonCurrentAccountsReceivable",
            "monthlyNonCurrentAccruedExpenses",
            "monthlyNonCurrentDeferredAssets",
            "monthlyNonCurrentDeferredLiabilities",
            "monthlyNonCurrentDeferredRevenue",
            "monthlyNonCurrentDeferredTaxesAssets",
            "monthlyNonCurrentDeferredTaxesLiabilities",
            "monthlyNonCurrentNoteReceivables",
            "monthlyNonCurrentPensionAndOtherPostretirementBenefitPlans",
            "monthlyNonCurrentPrepaidAssets",
            "monthlyNotesReceivable",
            "monthlyOrdinarySharesNumber",
            "monthlyOtherCapitalStock",
            "monthlyOtherCurrentAssets",
            "monthlyOtherCurrentBorrowings",
            "monthlyOtherCurrentLiabilities",
            "monthlyOtherEquityAdjustments",
            "monthlyOtherEquityInterest",
            "monthlyOtherIntangibleAssets",
            "monthlyOtherInventories",
            "monthlyOtherInvestments",
            "monthlyOtherNonCurrentAssets",
            "monthlyOtherNonCurrentLiabilities",
            "monthlyOtherPayable",
            "monthlyOtherProperties",
            "monthlyOtherReceivables",
            "monthlyOtherShortTermInvestments",
            "monthlyPayables",
            "monthlyPayablesAndAccruedExpenses",
            "monthlyPensionandOtherPostRetirementBenefitPlansCurrent",
            "monthlyPreferredSecuritiesOutsideStockEquity",
            "monthlyPreferredSharesNumber",
            "monthlyPreferredStock",
            "monthlyPreferredStockEquity",
            "monthlyPrepaidAssets",
            "monthlyProperties",
            "monthlyRawMaterials",
            "monthlyReceivables",
            "monthlyReceivablesAdjustmentsAllowances",
            "monthlyRestrictedCash",
            "monthlyRestrictedCommonStock",
            "monthlyRetainedEarnings",
            "monthlyShareIssued",
            "monthlyStockholdersEquity",
            "monthlyTangibleBookValue",
            "monthlyTaxesReceivable",
            "monthlyTotalAssets",
            "monthlyTotalCapitalization",
            "monthlyTotalDebt",
            "monthlyTotalEquityGrossMinorityInterest",
            "monthlyTotalLiabilitiesNetMinorityInterest",
            "monthlyTotalNonCurrentAssets",
            "monthlyTotalNonCurrentLiabilitiesNetMinorityInterest",
            "monthlyTotalPartnershipCapital",
            "monthlyTotalTaxPayable",
            "monthlyTradeandOtherPayablesNonCurrent",
            "monthlyTradingSecurities",
            "monthlyTreasurySharesNumber",
            "monthlyTreasuryStock",
            "monthlyUnrealizedGainLoss",
            "monthlyWorkInProcess",
            "monthlyWorkingCapital",
            "trailingAccountsPayable",
            "trailingAccountsReceivable",
            "trailingAccruedInterestReceivable",
            "trailingAccumulatedDepreciation",
            "trailingAdditionalPaidInCapital",
            "trailingAllowanceForDoubtfulAccountsReceivable",
            "trailingAssetsHeldForSaleCurrent",
            "trailingAvailableForSaleSecurities",
            "trailingBuildingsAndImprovements",
            "trailingCapitalLeaseObligations",
            "trailingCapitalStock",
            "trailingCashAndCashEquivalents",
            "trailingCashCashEquivalentsAndShortTermInvestments",
            "trailingCashEquivalents",
            "trailingCashFinancial",
            "trailingCommercialPaper",
            "trailingCommonStock",
            "trailingCommonStockEquity",
            "trailingConstructionInProgress",
            "trailingCurrentAccruedExpenses",
            "trailingCurrentAssets",
            "trailingCurrentCapitalLeaseObligation",
            "trailingCurrentDebt",
            "trailingCurrentDebtAndCapitalLeaseObligation",
            "trailingCurrentDeferredAssets",
            "trailingCurrentDeferredLiabilities",
            "trailingCurrentDeferredRevenue",
            "trailingCurrentDeferredTaxesAssets",
            "trailingCurrentDeferredTaxesLiabilities",
            "trailingCurrentLiabilities",
            "trailingCurrentNotesPayable",
            "trailingCurrentProvisions",
            "trailingDefinedPensionBenefit",
            "trailingDerivativeProductLiabilities",
            "trailingDividendsPayable",
            "trailingDuefromRelatedPartiesCurrent",
            "trailingDuefromRelatedPartiesNonCurrent",
            "trailingDuetoRelatedPartiesCurrent",
            "trailingDuetoRelatedPartiesNonCurrent",
            "trailingEmployeeBenefits",
            "trailingFinancialAssets",
            "trailingFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
            "trailingFinishedGoods",
            "trailingFixedAssetsRevaluationReserve",
            "trailingForeignCurrencyTranslationAdjustments",
            "trailingGainsLossesNotAffectingRetainedEarnings",
            "trailingGeneralPartnershipCapital",
            "trailingGoodwill",
            "trailingGoodwillAndOtherIntangibleAssets",
            "trailingGrossAccountsReceivable",
            "trailingGrossPPE",
            "trailingHedgingAssetsCurrent",
            "trailingHeldToMaturitySecurities",
            "trailingIncomeTaxPayable",
            "trailingInterestPayable",
            "trailingInventoriesAdjustmentsAllowances",
            "trailingInventory",
            "trailingInvestedCapital",
            "trailingInvestmentProperties",
            "trailingInvestmentinFinancialAssets",
            "trailingInvestmentsAndAdvances",
            "trailingInvestmentsInOtherVenturesUnderEquityMethod",
            "trailingInvestmentsinAssociatesatCost",
            "trailingInvestmentsinJointVenturesatCost",
            "trailingInvestmentsinSubsidiariesatCost",
            "trailingLandAndImprovements",
            "trailingLeases",
            "trailingLiabilitiesHeldforSaleNonCurrent",
            "trailingLimitedPartnershipCapital",
            "trailingLineOfCredit",
            "trailingLoansReceivable",
            "trailingLongTermCapitalLeaseObligation",
            "trailingLongTermDebt",
            "trailingLongTermDebtAndCapitalLeaseObligation",
            "trailingLongTermEquityInvestment",
            "trailingLongTermProvisions",
            "trailingMachineryFurnitureEquipment",
            "trailingMinimumPensionLiabilities",
            "trailingMinorityInterest",
            "trailingNetDebt",
            "trailingNetPPE",
            "trailingNetTangibleAssets",
            "trailingNonCurrentAccountsReceivable",
            "trailingNonCurrentAccruedExpenses",
            "trailingNonCurrentDeferredAssets",
            "trailingNonCurrentDeferredLiabilities",
            "trailingNonCurrentDeferredRevenue",
            "trailingNonCurrentDeferredTaxesAssets",
            "trailingNonCurrentDeferredTaxesLiabilities",
            "trailingNonCurrentNoteReceivables",
            "trailingNonCurrentPensionAndOtherPostretirementBenefitPlans",
            "trailingNonCurrentPrepaidAssets",
            "trailingNotesReceivable",
            "trailingOrdinarySharesNumber",
            "trailingOtherCapitalStock",
            "trailingOtherCurrentAssets",
            "trailingOtherCurrentBorrowings",
            "trailingOtherCurrentLiabilities",
            "trailingOtherEquityAdjustments",
            "trailingOtherEquityInterest",
            "trailingOtherIntangibleAssets",
            "trailingOtherInventories",
            "trailingOtherInvestments",
            "trailingOtherNonCurrentAssets",
            "trailingOtherNonCurrentLiabilities",
            "trailingOtherPayable",
            "trailingOtherProperties",
            "trailingOtherReceivables",
            "trailingOtherShortTermInvestments",
            "trailingPayables",
            "trailingPayablesAndAccruedExpenses",
            "trailingPensionandOtherPostRetirementBenefitPlansCurrent",
            "trailingPreferredSecuritiesOutsideStockEquity",
            "trailingPreferredSharesNumber",
            "trailingPreferredStock",
            "trailingPreferredStockEquity",
            "trailingPrepaidAssets",
            "trailingProperties",
            "trailingRawMaterials",
            "trailingReceivables",
            "trailingReceivablesAdjustmentsAllowances",
            "trailingRestrictedCash",
            "trailingRestrictedCommonStock",
            "trailingRetainedEarnings",
            "trailingShareIssued",
            "trailingStockholdersEquity",
            "trailingTangibleBookValue",
            "trailingTaxesReceivable",
            "trailingTotalAssets",
            "trailingTotalCapitalization",
            "trailingTotalDebt",
            "trailingTotalEquityGrossMinorityInterest",
            "trailingTotalLiabilitiesNetMinorityInterest",
            "trailingTotalNonCurrentAssets",
            "trailingTotalNonCurrentLiabilitiesNetMinorityInterest",
            "trailingTotalPartnershipCapital",
            "trailingTotalTaxPayable",
            "trailingTradeandOtherPayablesNonCurrent",
            "trailingTradingSecurities",
            "trailingTreasurySharesNumber",
            "trailingTreasuryStock",
            "trailingUnrealizedGainLoss",
            "trailingWorkInProcess",
            "trailingWorkingCapital",
        ]
    },
    "cash_flow": {
        "annual": [
            "annualAdjustedGeographySegmentData",
            "annualAmortizationCashFlow",
            "annualAmortizationOfIntangibles",
            "annualAmortizationOfSecurities",
            "annualAssetImpairmentCharge",
            "annualBeginningCashPosition",
            "annualCapitalExpenditure",
            "annualCapitalExpenditureReported",
            "annualCashDividendsPaid",
            "annualCashFlowFromContinuingFinancingActivities",
            "annualCashFlowFromContinuingInvestingActivities",
            "annualCashFlowFromContinuingOperatingActivities",
            "annualCashFlowFromDiscontinuedOperation",
            "annualCashFlowsfromusedinOperatingActivitiesDirect",
            "annualCashFromDiscontinuedFinancingActivities",
            "annualCashFromDiscontinuedInvestingActivities",
            "annualCashFromDiscontinuedOperatingActivities",
            "annualChangeInAccountPayable",
            "annualChangeInAccruedExpense",
            "annualChangeInCashSupplementalAsReported",
            "annualChangeInDividendPayable",
            "annualChangeInIncomeTaxPayable",
            "annualChangeInInterestPayable",
            "annualChangeInInventory",
            "annualChangeInOtherCurrentAssets",
            "annualChangeInOtherCurrentLiabilities",
            "annualChangeInOtherWorkingCapital",
            "annualChangeInPayable",
            "annualChangeInPayablesAndAccruedExpense",
            "annualChangeInPrepaidAssets",
            "annualChangeInReceivables",
            "annualChangeInTaxPayable",
            "annualChangeInWorkingCapital",
            "annualChangesInAccountReceivables",
            "annualChangesInCash",
            "annualClassesofCashPayments",
            "annualClassesofCashReceiptsfromOperatingActivities",
            "annualCommonStockDividendPaid",
            "annualCommonStockIssuance",
            "annualCommonStockPayments",
            "annualDeferredIncomeTax",
            "annualDeferredTax",
            "annualDepletion",
            "annualDepreciation",
            "annualDepreciationAmortizationDepletion",
            "annualDepreciationAndAmortization",
            "annualDividendPaidCFO",
            "annualDividendReceivedCFO",
            "annualDividendsPaidDirect",
            "annualDividendsReceivedCFI",
            "annualDividendsReceivedDirect",
            "annualDomesticSales",
            "annualEarningsLossesFromEquityInvestments",
            "annualEffectOfExchangeRateChanges",
            "annualEndCashPosition",
            "annualExcessTaxBenefitFromStockBasedCompensation",
            "annualFinancingCashFlow",
            "annualForeignSales",
            "annualFreeCashFlow",
            "annualGainLossOnInvestmentSecurities",
            "annualGainLossOnSaleOfBusiness",
            "annualGainLossOnSaleOfPPE",
            "annualIncomeTaxPaidSupplementalData",
            "annualInterestPaidCFF",
            "annualInterestPaidCFO",
            "annualInterestPaidDirect",
            "annualInterestPaidSupplementalData",
            "annualInterestReceivedCFI",
            "annualInterestReceivedCFO",
            "annualInterestReceivedDirect",
            "annualInvestingCashFlow",
            "annualIssuanceOfCapitalStock",
            "annualIssuanceOfDebt",
            "annualLongTermDebtIssuance",
            "annualLongTermDebtPayments",
            "annualNetBusinessPurchaseAndSale",
            "annualNetCommonStockIssuance",
            "annualNetForeignCurrencyExchangeGainLoss",
            "annualNetIncome",
            "annualNetIncomeFromContinuingOperations",
            "annualNetIntangiblesPurchaseAndSale",
            "annualNetInvestmentPropertiesPurchaseAndSale",
            "annualNetInvestmentPurchaseAndSale",
            "annualNetIssuancePaymentsOfDebt",
            "annualNetLongTermDebtIssuance",
            "annualNetOtherFinancingCharges",
            "annualNetOtherInvestingChanges",
            "annualNetPPEPurchaseAndSale",
            "annualNetPreferredStockIssuance",
            "annualNetShortTermDebtIssuance",
            "annualOperatingCashFlow",
            "annualOperatingGainsLosses",
            "annualOtherCashAdjustmentInsideChangeinCash",
            "annualOtherCashAdjustmentOutsideChangeinCash",
            "annualOtherCashPaymentsfromOperatingActivities",
            "annualOtherCashReceiptsfromOperatingActivities",
            "annualOtherNonCashItems",
            "annualPaymentsonBehalfofEmployees",
            "annualPaymentstoSuppliersforGoodsandServices",
            "annualPensionAndEmployeeBenefitExpense",
            "annualPreferredStockDividendPaid",
            "annualPreferredStockIssuance",
            "annualPreferredStockPayments",
            "annualProceedsFromStockOptionExercised",
            "annualProvisionandWriteOffofAssets",
            "annualPurchaseOfBusiness",
            "annualPurchaseOfIntangibles",
            "annualPurchaseOfInvestment",
            "annualPurchaseOfInvestmentProperties",
            "annualPurchaseOfPPE",
            "annualReceiptsfromCustomers",
            "annualReceiptsfromGovernmentGrants",
            "annualRepaymentOfDebt",
            "annualRepurchaseOfCapitalStock",
            "annualSaleOfBusiness",
            "annualSaleOfIntangibles",
            "annualSaleOfInvestment",
            "annualSaleOfInvestmentProperties",
            "annualSaleOfPPE",
            "annualShortTermDebtIssuance",
            "annualShortTermDebtPayments",
            "annualStockBasedCompensation",
            "annualTaxesRefundPaid",
            "annualTaxesRefundPaidDirect",
            "annualUnrealizedGainLossOnInvestmentSecurities",
            "trailingAdjustedGeographySegmentData",
            "trailingAmortizationCashFlow",
            "trailingAmortizationOfIntangibles",
            "trailingAmortizationOfSecurities",
            "trailingAssetImpairmentCharge",
            "trailingBeginningCashPosition",
            "trailingCapitalExpenditure",
            "trailingCapitalExpenditureReported",
            "trailingCashDividendsPaid",
            "trailingCashFlowFromContinuingFinancingActivities",
            "trailingCashFlowFromContinuingInvestingActivities",
            "trailingCashFlowFromContinuingOperatingActivities",
            "trailingCashFlowFromDiscontinuedOperation",
            "trailingCashFlowsfromusedinOperatingActivitiesDirect",
            "trailingCashFromDiscontinuedFinancingActivities",
            "trailingCashFromDiscontinuedInvestingActivities",
            "trailingCashFromDiscontinuedOperatingActivities",
            "trailingChangeInAccountPayable",
            "trailingChangeInAccruedExpense",
            "trailingChangeInCashSupplementalAsReported",
            "trailingChangeInDividendPayable",
            "trailingChangeInIncomeTaxPayable",
            "trailingChangeInInterestPayable",
            "trailingChangeInInventory",
            "trailingChangeInOtherCurrentAssets",
            "trailingChangeInOtherCurrentLiabilities",
            "trailingChangeInOtherWorkingCapital",
            "trailingChangeInPayable",
            "trailingChangeInPayablesAndAccruedExpense",
            "trailingChangeInPrepaidAssets",
            "trailingChangeInReceivables",
            "trailingChangeInTaxPayable",
            "trailingChangeInWorkingCapital",
            "trailingChangesInAccountReceivables",
            "trailingChangesInCash",
            "trailingClassesofCashPayments",
            "trailingClassesofCashReceiptsfromOperatingActivities",
            "trailingCommonStockDividendPaid",
            "trailingCommonStockIssuance",
            "trailingCommonStockPayments",
            "trailingDeferredIncomeTax",
            "trailingDeferredTax",
            "trailingDepletion",
            "trailingDepreciation",
            "trailingDepreciationAmortizationDepletion",
            "trailingDepreciationAndAmortization",
            "trailingDividendPaidCFO",
            "trailingDividendReceivedCFO",
            "trailingDividendsPaidDirect",
            "trailingDividendsReceivedCFI",
            "trailingDividendsReceivedDirect",
            "trailingDomesticSales",
            "trailingEarningsLossesFromEquityInvestments",
            "trailingEffectOfExchangeRateChanges",
            "trailingEndCashPosition",
            "trailingExcessTaxBenefitFromStockBasedCompensation",
            "trailingFinancingCashFlow",
            "trailingForeignSales",
            "trailingFreeCashFlow",
            "trailingGainLossOnInvestmentSecurities",
            "trailingGainLossOnSaleOfBusiness",
            "trailingGainLossOnSaleOfPPE",
            "trailingIncomeTaxPaidSupplementalData",
            "trailingInterestPaidCFF",
            "trailingInterestPaidCFO",
            "trailingInterestPaidDirect",
            "trailingInterestPaidSupplementalData",
            "trailingInterestReceivedCFI",
            "trailingInterestReceivedCFO",
            "trailingInterestReceivedDirect",
            "trailingInvestingCashFlow",
            "trailingIssuanceOfCapitalStock",
            "trailingIssuanceOfDebt",
            "trailingLongTermDebtIssuance",
            "trailingLongTermDebtPayments",
            "trailingNetBusinessPurchaseAndSale",
            "trailingNetCommonStockIssuance",
            "trailingNetForeignCurrencyExchangeGainLoss",
            "trailingNetIncome",
            "trailingNetIncomeFromContinuingOperations",
            "trailingNetIntangiblesPurchaseAndSale",
            "trailingNetInvestmentPropertiesPurchaseAndSale",
            "trailingNetInvestmentPurchaseAndSale",
            "trailingNetIssuancePaymentsOfDebt",
            "trailingNetLongTermDebtIssuance",
            "trailingNetOtherFinancingCharges",
            "trailingNetOtherInvestingChanges",
            "trailingNetPPEPurchaseAndSale",
            "trailingNetPreferredStockIssuance",
            "trailingNetShortTermDebtIssuance",
            "trailingOperatingCashFlow",
            "trailingOperatingGainsLosses",
            "trailingOtherCashAdjustmentInsideChangeinCash",
            "trailingOtherCashAdjustmentOutsideChangeinCash",
            "trailingOtherCashPaymentsfromOperatingActivities",
            "trailingOtherCashReceiptsfromOperatingActivities",
            "trailingOtherNonCashItems",
            "trailingPaymentsonBehalfofEmployees",
            "trailingPaymentstoSuppliersforGoodsandServices",
            "trailingPensionAndEmployeeBenefitExpense",
            "trailingPreferredStockDividendPaid",
            "trailingPreferredStockIssuance",
            "trailingPreferredStockPayments",
            "trailingProceedsFromStockOptionExercised",
            "trailingProvisionandWriteOffofAssets",
            "trailingPurchaseOfBusiness",
            "trailingPurchaseOfIntangibles",
            "trailingPurchaseOfInvestment",
            "trailingPurchaseOfInvestmentProperties",
            "trailingPurchaseOfPPE",
            "trailingReceiptsfromCustomers",
            "trailingReceiptsfromGovernmentGrants",
            "trailingRepaymentOfDebt",
            "trailingRepurchaseOfCapitalStock",
            "trailingSaleOfBusiness",
            "trailingSaleOfIntangibles",
            "trailingSaleOfInvestment",
            "trailingSaleOfInvestmentProperties",
            "trailingSaleOfPPE",
            "trailingShortTermDebtIssuance",
            "trailingShortTermDebtPayments",
            "trailingStockBasedCompensation",
            "trailingTaxesRefundPaid",
            "trailingTaxesRefundPaidDirect",
            "trailingUnrealizedGainLossOnInvestmentSecurities",
        ],
        "quarterly": [
            "quarterlyAdjustedGeographySegmentData",
            "quarterlyAmortizationCashFlow",
            "quarterlyAmortizationOfIntangibles",
            "quarterlyAmortizationOfSecurities",
            "quarterlyAssetImpairmentCharge",
            "quarterlyBeginningCashPosition",
            "quarterlyCapitalExpenditure",
            "quarterlyCapitalExpenditureReported",
            "quarterlyCashDividendsPaid",
            "quarterlyCashFlowFromContinuingFinancingActivities",
            "quarterlyCashFlowFromContinuingInvestingActivities",
            "quarterlyCashFlowFromContinuingOperatingActivities",
            "quarterlyCashFlowFromDiscontinuedOperation",
            "quarterlyCashFlowsfromusedinOperatingActivitiesDirect",
            "quarterlyCashFromDiscontinuedFinancingActivities",
            "quarterlyCashFromDiscontinuedInvestingActivities",
            "quarterlyCashFromDiscontinuedOperatingActivities",
            "quarterlyChangeInAccountPayable",
            "quarterlyChangeInAccruedExpense",
            "quarterlyChangeInCashSupplementalAsReported",
            "quarterlyChangeInDividendPayable",
            "quarterlyChangeInIncomeTaxPayable",
            "quarterlyChangeInInterestPayable",
            "quarterlyChangeInInventory",
            "quarterlyChangeInOtherCurrentAssets",
            "quarterlyChangeInOtherCurrentLiabilities",
            "quarterlyChangeInOtherWorkingCapital",
            "quarterlyChangeInPayable",
            "quarterlyChangeInPayablesAndAccruedExpense",
            "quarterlyChangeInPrepaidAssets",
            "quarterlyChangeInReceivables",
            "quarterlyChangeInTaxPayable",
            "quarterlyChangeInWorkingCapital",
            "quarterlyChangesInAccountReceivables",
            "quarterlyChangesInCash",
            "quarterlyClassesofCashPayments",
            "quarterlyClassesofCashReceiptsfromOperatingActivities",
            "quarterlyCommonStockDividendPaid",
            "quarterlyCommonStockIssuance",
            "quarterlyCommonStockPayments",
            "quarterlyDeferredIncomeTax",
            "quarterlyDeferredTax",
            "quarterlyDepletion",
            "quarterlyDepreciation",
            "quarterlyDepreciationAmortizationDepletion",
            "quarterlyDepreciationAndAmortization",
            "quarterlyDividendPaidCFO",
            "quarterlyDividendReceivedCFO",
            "quarterlyDividendsPaidDirect",
            "quarterlyDividendsReceivedCFI",
            "quarterlyDividendsReceivedDirect",
            "quarterlyDomesticSales",
            "quarterlyEarningsLossesFromEquityInvestments",
            "quarterlyEffectOfExchangeRateChanges",
            "quarterlyEndCashPosition",
            "quarterlyExcessTaxBenefitFromStockBasedCompensation",
            "quarterlyFinancingCashFlow",
            "quarterlyForeignSales",
            "quarterlyFreeCashFlow",
            "quarterlyGainLossOnInvestmentSecurities",
            "quarterlyGainLossOnSaleOfBusiness",
            "quarterlyGainLossOnSaleOfPPE",
            "quarterlyIncomeTaxPaidSupplementalData",
            "quarterlyInterestPaidCFF",
            "quarterlyInterestPaidCFO",
            "quarterlyInterestPaidDirect",
            "quarterlyInterestPaidSupplementalData",
            "quarterlyInterestReceivedCFI",
            "quarterlyInterestReceivedCFO",
            "quarterlyInterestReceivedDirect",
            "quarterlyInvestingCashFlow",
            "quarterlyIssuanceOfCapitalStock",
            "quarterlyIssuanceOfDebt",
            "quarterlyLongTermDebtIssuance",
            "quarterlyLongTermDebtPayments",
            "quarterlyNetBusinessPurchaseAndSale",
            "quarterlyNetCommonStockIssuance",
            "quarterlyNetForeignCurrencyExchangeGainLoss",
            "quarterlyNetIncome",
            "quarterlyNetIncomeFromContinuingOperations",
            "quarterlyNetIntangiblesPurchaseAndSale",
            "quarterlyNetInvestmentPropertiesPurchaseAndSale",
            "quarterlyNetInvestmentPurchaseAndSale",
            "quarterlyNetIssuancePaymentsOfDebt",
            "quarterlyNetLongTermDebtIssuance",
            "quarterlyNetOtherFinancingCharges",
            "quarterlyNetOtherInvestingChanges",
            "quarterlyNetPPEPurchaseAndSale",
            "quarterlyNetPreferredStockIssuance",
            "quarterlyNetShortTermDebtIssuance",
            "quarterlyOperatingCashFlow",
            "quarterlyOperatingGainsLosses",
            "quarterlyOtherCashAdjustmentInsideChangeinCash",
            "quarterlyOtherCashAdjustmentOutsideChangeinCash",
            "quarterlyOtherCashPaymentsfromOperatingActivities",
            "quarterlyOtherCashReceiptsfromOperatingActivities",
            "quarterlyOtherNonCashItems",
            "quarterlyPaymentsonBehalfofEmployees",
            "quarterlyPaymentstoSuppliersforGoodsandServices",
            "quarterlyPensionAndEmployeeBenefitExpense",
            "quarterlyPreferredStockDividendPaid",
            "quarterlyPreferredStockIssuance",
            "quarterlyPreferredStockPayments",
            "quarterlyProceedsFromStockOptionExercised",
            "quarterlyProvisionandWriteOffofAssets",
            "quarterlyPurchaseOfBusiness",
            "quarterlyPurchaseOfIntangibles",
            "quarterlyPurchaseOfInvestment",
            "quarterlyPurchaseOfInvestmentProperties",
            "quarterlyPurchaseOfPPE",
            "quarterlyReceiptsfromCustomers",
            "quarterlyReceiptsfromGovernmentGrants",
            "quarterlyRepaymentOfDebt",
            "quarterlyRepurchaseOfCapitalStock",
            "quarterlySaleOfBusiness",
            "quarterlySaleOfIntangibles",
            "quarterlySaleOfInvestment",
            "quarterlySaleOfInvestmentProperties",
            "quarterlySaleOfPPE",
            "quarterlyShortTermDebtIssuance",
            "quarterlyShortTermDebtPayments",
            "quarterlyStockBasedCompensation",
            "quarterlyTaxesRefundPaid",
            "quarterlyTaxesRefundPaidDirect",
            "quarterlyUnrealizedGainLossOnInvestmentSecurities",
            "trailingAdjustedGeographySegmentData",
            "trailingAmortizationCashFlow",
            "trailingAmortizationOfIntangibles",
            "trailingAmortizationOfSecurities",
            "trailingAssetImpairmentCharge",
            "trailingBeginningCashPosition",
            "trailingCapitalExpenditure",
            "trailingCapitalExpenditureReported",
            "trailingCashDividendsPaid",
            "trailingCashFlowFromContinuingFinancingActivities",
            "trailingCashFlowFromContinuingInvestingActivities",
            "trailingCashFlowFromContinuingOperatingActivities",
            "trailingCashFlowFromDiscontinuedOperation",
            "trailingCashFlowsfromusedinOperatingActivitiesDirect",
            "trailingCashFromDiscontinuedFinancingActivities",
            "trailingCashFromDiscontinuedInvestingActivities",
            "trailingCashFromDiscontinuedOperatingActivities",
            "trailingChangeInAccountPayable",
            "trailingChangeInAccruedExpense",
            "trailingChangeInCashSupplementalAsReported",
            "trailingChangeInDividendPayable",
            "trailingChangeInIncomeTaxPayable",
            "trailingChangeInInterestPayable",
            "trailingChangeInInventory",
            "trailingChangeInOtherCurrentAssets",
            "trailingChangeInOtherCurrentLiabilities",
            "trailingChangeInOtherWorkingCapital",
            "trailingChangeInPayable",
            "trailingChangeInPayablesAndAccruedExpense",
            "trailingChangeInPrepaidAssets",
            "trailingChangeInReceivables",
            "trailingChangeInTaxPayable",
            "trailingChangeInWorkingCapital",
            "trailingChangesInAccountReceivables",
            "trailingChangesInCash",
            "trailingClassesofCashPayments",
            "trailingClassesofCashReceiptsfromOperatingActivities",
            "trailingCommonStockDividendPaid",
            "trailingCommonStockIssuance",
            "trailingCommonStockPayments",
            "trailingDeferredIncomeTax",
            "trailingDeferredTax",
            "trailingDepletion",
            "trailingDepreciation",
            "trailingDepreciationAmortizationDepletion",
            "trailingDepreciationAndAmortization",
            "trailingDividendPaidCFO",
            "trailingDividendReceivedCFO",
            "trailingDividendsPaidDirect",
            "trailingDividendsReceivedCFI",
            "trailingDividendsReceivedDirect",
            "trailingDomesticSales",
            "trailingEarningsLossesFromEquityInvestments",
            "trailingEffectOfExchangeRateChanges",
            "trailingEndCashPosition",
            "trailingExcessTaxBenefitFromStockBasedCompensation",
            "trailingFinancingCashFlow",
            "trailingForeignSales",
            "trailingFreeCashFlow",
            "trailingGainLossOnInvestmentSecurities",
            "trailingGainLossOnSaleOfBusiness",
            "trailingGainLossOnSaleOfPPE",
            "trailingIncomeTaxPaidSupplementalData",
            "trailingInterestPaidCFF",
            "trailingInterestPaidCFO",
            "trailingInterestPaidDirect",
            "trailingInterestPaidSupplementalData",
            "trailingInterestReceivedCFI",
            "trailingInterestReceivedCFO",
            "trailingInterestReceivedDirect",
            "trailingInvestingCashFlow",
            "trailingIssuanceOfCapitalStock",
            "trailingIssuanceOfDebt",
            "trailingLongTermDebtIssuance",
            "trailingLongTermDebtPayments",
            "trailingNetBusinessPurchaseAndSale",
            "trailingNetCommonStockIssuance",
            "trailingNetForeignCurrencyExchangeGainLoss",
            "trailingNetIncome",
            "trailingNetIncomeFromContinuingOperations",
            "trailingNetIntangiblesPurchaseAndSale",
            "trailingNetInvestmentPropertiesPurchaseAndSale",
            "trailingNetInvestmentPurchaseAndSale",
            "trailingNetIssuancePaymentsOfDebt",
            "trailingNetLongTermDebtIssuance",
            "trailingNetOtherFinancingCharges",
            "trailingNetOtherInvestingChanges",
            "trailingNetPPEPurchaseAndSale",
            "trailingNetPreferredStockIssuance",
            "trailingNetShortTermDebtIssuance",
            "trailingOperatingCashFlow",
            "trailingOperatingGainsLosses",
            "trailingOtherCashAdjustmentInsideChangeinCash",
            "trailingOtherCashAdjustmentOutsideChangeinCash",
            "trailingOtherCashPaymentsfromOperatingActivities",
            "trailingOtherCashReceiptsfromOperatingActivities",
            "trailingOtherNonCashItems",
            "trailingPaymentsonBehalfofEmployees",
            "trailingPaymentstoSuppliersforGoodsandServices",
            "trailingPensionAndEmployeeBenefitExpense",
            "trailingPreferredStockDividendPaid",
            "trailingPreferredStockIssuance",
            "trailingPreferredStockPayments",
            "trailingProceedsFromStockOptionExercised",
            "trailingProvisionandWriteOffofAssets",
            "trailingPurchaseOfBusiness",
            "trailingPurchaseOfIntangibles",
            "trailingPurchaseOfInvestment",
            "trailingPurchaseOfInvestmentProperties",
            "trailingPurchaseOfPPE",
            "trailingReceiptsfromCustomers",
            "trailingReceiptsfromGovernmentGrants",
            "trailingRepaymentOfDebt",
            "trailingRepurchaseOfCapitalStock",
            "trailingSaleOfBusiness",
            "trailingSaleOfIntangibles",
            "trailingSaleOfInvestment",
            "trailingSaleOfInvestmentProperties",
            "trailingSaleOfPPE",
            "trailingShortTermDebtIssuance",
            "trailingShortTermDebtPayments",
            "trailingStockBasedCompensation",
            "trailingTaxesRefundPaid",
            "trailingTaxesRefundPaidDirect",
            "trailingUnrealizedGainLossOnInvestmentSecurities",
        ],
        "monthly": [
            "monthlyAdjustedGeographySegmentData",
            "monthlyAmortizationCashFlow",
            "monthlyAmortizationOfIntangibles",
            "monthlyAmortizationOfSecurities",
            "monthlyAssetImpairmentCharge",
            "monthlyBeginningCashPosition",
            "monthlyCapitalExpenditure",
            "monthlyCapitalExpenditureReported",
            "monthlyCashDividendsPaid",
            "monthlyCashFlowFromContinuingFinancingActivities",
            "monthlyCashFlowFromContinuingInvestingActivities",
            "monthlyCashFlowFromContinuingOperatingActivities",
            "monthlyCashFlowFromDiscontinuedOperation",
            "monthlyCashFlowsfromusedinOperatingActivitiesDirect",
            "monthlyCashFromDiscontinuedFinancingActivities",
            "monthlyCashFromDiscontinuedInvestingActivities",
            "monthlyCashFromDiscontinuedOperatingActivities",
            "monthlyChangeInAccountPayable",
            "monthlyChangeInAccruedExpense",
            "monthlyChangeInCashSupplementalAsReported",
            "monthlyChangeInDividendPayable",
            "monthlyChangeInIncomeTaxPayable",
            "monthlyChangeInInterestPayable",
            "monthlyChangeInInventory",
            "monthlyChangeInOtherCurrentAssets",
            "monthlyChangeInOtherCurrentLiabilities",
            "monthlyChangeInOtherWorkingCapital",
            "monthlyChangeInPayable",
            "monthlyChangeInPayablesAndAccruedExpense",
            "monthlyChangeInPrepaidAssets",
            "monthlyChangeInReceivables",
            "monthlyChangeInTaxPayable",
            "monthlyChangeInWorkingCapital",
            "monthlyChangesInAccountReceivables",
            "monthlyChangesInCash",
            "monthlyClassesofCashPayments",
            "monthlyClassesofCashReceiptsfromOperatingActivities",
            "monthlyCommonStockDividendPaid",
            "monthlyCommonStockIssuance",
            "monthlyCommonStockPayments",
            "monthlyDeferredIncomeTax",
            "monthlyDeferredTax",
            "monthlyDepletion",
            "monthlyDepreciation",
            "monthlyDepreciationAmortizationDepletion",
            "monthlyDepreciationAndAmortization",
            "monthlyDividendPaidCFO",
            "monthlyDividendReceivedCFO",
            "monthlyDividendsPaidDirect",
            "monthlyDividendsReceivedCFI",
            "monthlyDividendsReceivedDirect",
            "monthlyDomesticSales",
            "monthlyEarningsLossesFromEquityInvestments",
            "monthlyEffectOfExchangeRateChanges",
            "monthlyEndCashPosition",
            "monthlyExcessTaxBenefitFromStockBasedCompensation",
            "monthlyFinancingCashFlow",
            "monthlyForeignSales",
            "monthlyFreeCashFlow",
            "monthlyGainLossOnInvestmentSecurities",
            "monthlyGainLossOnSaleOfBusiness",
            "monthlyGainLossOnSaleOfPPE",
            "monthlyIncomeTaxPaidSupplementalData",
            "monthlyInterestPaidCFF",
            "monthlyInterestPaidCFO",
            "monthlyInterestPaidDirect",
            "monthlyInterestPaidSupplementalData",
            "monthlyInterestReceivedCFI",
            "monthlyInterestReceivedCFO",
            "monthlyInterestReceivedDirect",
            "monthlyInvestingCashFlow",
            "monthlyIssuanceOfCapitalStock",
            "monthlyIssuanceOfDebt",
            "monthlyLongTermDebtIssuance",
            "monthlyLongTermDebtPayments",
            "monthlyNetBusinessPurchaseAndSale",
            "monthlyNetCommonStockIssuance",
            "monthlyNetForeignCurrencyExchangeGainLoss",
            "monthlyNetIncome",
            "monthlyNetIncomeFromContinuingOperations",
            "monthlyNetIntangiblesPurchaseAndSale",
            "monthlyNetInvestmentPropertiesPurchaseAndSale",
            "monthlyNetInvestmentPurchaseAndSale",
            "monthlyNetIssuancePaymentsOfDebt",
            "monthlyNetLongTermDebtIssuance",
            "monthlyNetOtherFinancingCharges",
            "monthlyNetOtherInvestingChanges",
            "monthlyNetPPEPurchaseAndSale",
            "monthlyNetPreferredStockIssuance",
            "monthlyNetShortTermDebtIssuance",
            "monthlyOperatingCashFlow",
            "monthlyOperatingGainsLosses",
            "monthlyOtherCashAdjustmentInsideChangeinCash",
            "monthlyOtherCashAdjustmentOutsideChangeinCash",
            "monthlyOtherCashPaymentsfromOperatingActivities",
            "monthlyOtherCashReceiptsfromOperatingActivities",
            "monthlyOtherNonCashItems",
            "monthlyPaymentsonBehalfofEmployees",
            "monthlyPaymentstoSuppliersforGoodsandServices",
            "monthlyPensionAndEmployeeBenefitExpense",
            "monthlyPreferredStockDividendPaid",
            "monthlyPreferredStockIssuance",
            "monthlyPreferredStockPayments",
            "monthlyProceedsFromStockOptionExercised",
            "monthlyProvisionandWriteOffofAssets",
            "monthlyPurchaseOfBusiness",
            "monthlyPurchaseOfIntangibles",
            "monthlyPurchaseOfInvestment",
            "monthlyPurchaseOfInvestmentProperties",
            "monthlyPurchaseOfPPE",
            "monthlyReceiptsfromCustomers",
            "monthlyReceiptsfromGovernmentGrants",
            "monthlyRepaymentOfDebt",
            "monthlyRepurchaseOfCapitalStock",
            "monthlySaleOfBusiness",
            "monthlySaleOfIntangibles",
            "monthlySaleOfInvestment",
            "monthlySaleOfInvestmentProperties",
            "monthlySaleOfPPE",
            "monthlyShortTermDebtIssuance",
            "monthlyShortTermDebtPayments",
            "monthlyStockBasedCompensation",
            "monthlyTaxesRefundPaid",
            "monthlyTaxesRefundPaidDirect",
            "monthlyUnrealizedGainLossOnInvestmentSecurities",
            "trailingAdjustedGeographySegmentData",
            "trailingAmortizationCashFlow",
            "trailingAmortizationOfIntangibles",
            "trailingAmortizationOfSecurities",
            "trailingAssetImpairmentCharge",
            "trailingBeginningCashPosition",
            "trailingCapitalExpenditure",
            "trailingCapitalExpenditureReported",
            "trailingCashDividendsPaid",
            "trailingCashFlowFromContinuingFinancingActivities",
            "trailingCashFlowFromContinuingInvestingActivities",
            "trailingCashFlowFromContinuingOperatingActivities",
            "trailingCashFlowFromDiscontinuedOperation",
            "trailingCashFlowsfromusedinOperatingActivitiesDirect",
            "trailingCashFromDiscontinuedFinancingActivities",
            "trailingCashFromDiscontinuedInvestingActivities",
            "trailingCashFromDiscontinuedOperatingActivities",
            "trailingChangeInAccountPayable",
            "trailingChangeInAccruedExpense",
            "trailingChangeInCashSupplementalAsReported",
            "trailingChangeInDividendPayable",
            "trailingChangeInIncomeTaxPayable",
            "trailingChangeInInterestPayable",
            "trailingChangeInInventory",
            "trailingChangeInOtherCurrentAssets",
            "trailingChangeInOtherCurrentLiabilities",
            "trailingChangeInOtherWorkingCapital",
            "trailingChangeInPayable",
            "trailingChangeInPayablesAndAccruedExpense",
            "trailingChangeInPrepaidAssets",
            "trailingChangeInReceivables",
            "trailingChangeInTaxPayable",
            "trailingChangeInWorkingCapital",
            "trailingChangesInAccountReceivables",
            "trailingChangesInCash",
            "trailingClassesofCashPayments",
            "trailingClassesofCashReceiptsfromOperatingActivities",
            "trailingCommonStockDividendPaid",
            "trailingCommonStockIssuance",
            "trailingCommonStockPayments",
            "trailingDeferredIncomeTax",
            "trailingDeferredTax",
            "trailingDepletion",
            "trailingDepreciation",
            "trailingDepreciationAmortizationDepletion",
            "trailingDepreciationAndAmortization",
            "trailingDividendPaidCFO",
            "trailingDividendReceivedCFO",
            "trailingDividendsPaidDirect",
            "trailingDividendsReceivedCFI",
            "trailingDividendsReceivedDirect",
            "trailingDomesticSales",
            "trailingEarningsLossesFromEquityInvestments",
            "trailingEffectOfExchangeRateChanges",
            "trailingEndCashPosition",
            "trailingExcessTaxBenefitFromStockBasedCompensation",
            "trailingFinancingCashFlow",
            "trailingForeignSales",
            "trailingFreeCashFlow",
            "trailingGainLossOnInvestmentSecurities",
            "trailingGainLossOnSaleOfBusiness",
            "trailingGainLossOnSaleOfPPE",
            "trailingIncomeTaxPaidSupplementalData",
            "trailingInterestPaidCFF",
            "trailingInterestPaidCFO",
            "trailingInterestPaidDirect",
            "trailingInterestPaidSupplementalData",
            "trailingInterestReceivedCFI",
            "trailingInterestReceivedCFO",
            "trailingInterestReceivedDirect",
            "trailingInvestingCashFlow",
            "trailingIssuanceOfCapitalStock",
            "trailingIssuanceOfDebt",
            "trailingLongTermDebtIssuance",
            "trailingLongTermDebtPayments",
            "trailingNetBusinessPurchaseAndSale",
            "trailingNetCommonStockIssuance",
            "trailingNetForeignCurrencyExchangeGainLoss",
            "trailingNetIncome",
            "trailingNetIncomeFromContinuingOperations",
            "trailingNetIntangiblesPurchaseAndSale",
            "trailingNetInvestmentPropertiesPurchaseAndSale",
            "trailingNetInvestmentPurchaseAndSale",
            "trailingNetIssuancePaymentsOfDebt",
            "trailingNetLongTermDebtIssuance",
            "trailingNetOtherFinancingCharges",
            "trailingNetOtherInvestingChanges",
            "trailingNetPPEPurchaseAndSale",
            "trailingNetPreferredStockIssuance",
            "trailingNetShortTermDebtIssuance",
            "trailingOperatingCashFlow",
            "trailingOperatingGainsLosses",
            "trailingOtherCashAdjustmentInsideChangeinCash",
            "trailingOtherCashAdjustmentOutsideChangeinCash",
            "trailingOtherCashPaymentsfromOperatingActivities",
            "trailingOtherCashReceiptsfromOperatingActivities",
            "trailingOtherNonCashItems",
            "trailingPaymentsonBehalfofEmployees",
            "trailingPaymentstoSuppliersforGoodsandServices",
            "trailingPensionAndEmployeeBenefitExpense",
            "trailingPreferredStockDividendPaid",
            "trailingPreferredStockIssuance",
            "trailingPreferredStockPayments",
            "trailingProceedsFromStockOptionExercised",
            "trailingProvisionandWriteOffofAssets",
            "trailingPurchaseOfBusiness",
            "trailingPurchaseOfIntangibles",
            "trailingPurchaseOfInvestment",
            "trailingPurchaseOfInvestmentProperties",
            "trailingPurchaseOfPPE",
            "trailingReceiptsfromCustomers",
            "trailingReceiptsfromGovernmentGrants",
            "trailingRepaymentOfDebt",
            "trailingRepurchaseOfCapitalStock",
            "trailingSaleOfBusiness",
            "trailingSaleOfIntangibles",
            "trailingSaleOfInvestment",
            "trailingSaleOfInvestmentProperties",
            "trailingSaleOfPPE",
            "trailingShortTermDebtIssuance",
            "trailingShortTermDebtPayments",
            "trailingStockBasedCompensation",
            "trailingTaxesRefundPaid",
            "trailingTaxesRefundPaidDirect",
            "trailingUnrealizedGainLossOnInvestmentSecurities",
        ]
    },
    "valuation": {
        "quarterly": [
            "quarterlyForwardPeRatio",
            "quarterlyPsRatio",
            "quarterlyPbRatio",
            "quarterlyEnterprisesValueEBITDARatio",
            "quarterlyEnterprisesValueRevenueRatio",
            "quarterlyPeRatio",
            "quarterlyMarketCap",
            "quarterlyEnterpriseValue",
            "quarterlyPegRatio",
            "trailingForwardPeRatio",
            "trailingPsRatio",
            "trailingPbRatio",
            "trailingEnterprisesValueEBITDARatio",
            "trailingEnterprisesValueRevenueRatio",
            "trailingPeRatio",
            "trailingMarketCap",
            "trailingEnterpriseValue",
            "trailingPegRatio",
        ],
    }
}

FUNDAMENTALS_TIME_MAP = {
    "annual": {"prefix": "annual", "period_type": "12M"},
    "quarterly": {"prefix": "quarterly", "period_type": "3M"},
    "monthly": {"prefix": "monthly", "period_type": "1M"},
}

MODULES_MAP = {
    "assetProfile": {
        "convert_dates": ["governanceEpochDate", "compensationAsOfEpochDate"]
    },
    "balanceSheetHistory": {
        "filter": "balanceSheetStatements",
        "convert_dates": ["endDate"],
    },
    "balanceSheetHistoryQuarterly": {
        "filter": "balanceSheetStatements",
        "convert_dates": ["endDate"],
    },
    "calendarEvents": {
        "convert_dates": ["earningsDate", "exDividendDate", "dividendDate"]
    },
    "cashflowStatementHistory": {
        "filter": "cashflowStatements",
        "convert_dates": ["endDate"],
    },
    "cashflowStatementHistoryQuarterly": {
        "filter": "cashflowStatements",
        "convert_dates": ["endDate"],
    },
    "defaultKeyStatistics": {
        "convert_dates": [
            "sharesShortPreviousMonthDate",
            "dateShortInterest",
            "lastFiscalYearEnd",
            "nextFiscalYearEnd",
            "fundInceptionDate",
            "lastSplitDate",
            "mostRecentQuarter",
        ]
    },
    "earnings": {"convert_dates": ["earningsDate"]},
    "earningsHistory": {"filter": "history", "convert_dates": ["quarter"]},
    "earningsTrend": {"convert_dates": []},
    "esgScores": {"convert_dates": []},
    "financialData": {"convert_dates": []},
    "fundOwnership": {"filter": "ownershipList", "convert_dates": ["reportDate"]},
    "fundPerformance": {"convert_dates": ["asOfDate"]},
    "fundProfile": {"convert_dates": []},
    "indexTrend": {"convert_dates": []},
    "incomeStatementHistory": {
        "filter": "incomeStatementHistory",
        "convert_dates": ["endDate"],
    },
    "incomeStatementHistoryQuarterly": {
        "filter": "incomeStatementHistory",
        "convert_dates": ["endDate"],
    },
    "industryTrend": {"convert_dates": []},
    "insiderHolders": {
        "filter": "holders",
        "convert_dates": ["latestTransDate", "positionDirectDate"],
    },
    "insiderTransactions": {
        "filter": "transactions",
        "convert_dates": ["startDate"],
    },
    "institutionOwnership": {
        "filter": "ownershipList",
        "convert_dates": ["reportDate"],
    },
    "majorHoldersBreakdown": {"convert_dates": []},
    "pageViews": {"convert_dates": []},
    "price": {"convert_dates": [
        "postMarketTime", "preMarketTime", "regularMarketTime"
    ]},
    "quoteType": {"convert_dates": ["firstTradeDateEpochUtc"]},
    "recommendationTrend": {"filter": "trend", "convert_dates": []},
    "secFilings": {"filter": "filings", "convert_dates": ["epochDate"]},
    "netSharePurchaseActivity": {"convert_dates": []},
    "sectorTrend": {"convert_dates": []},
    "summaryDetail": {
        "convert_dates": ["exDividendDate", "expireDate", "startDate"]
    },
    "summaryProfile": {"convert_dates": []},
    "topHoldings": {"convert_dates": []},
    "upgradeDowngradeHistory": {
        "filter": "history",
        "convert_dates": ["epochGradeDate"],
    },
}

REQUEST_MAP = {
    "quoteSummary": {
        "path": "https://query2.finance.yahoo.com/v10/finance/quoteSummary/{symbol}",
        "response_field": "quoteSummary",
        "request": {
            "formatted": {"required": False, "default": False},
            "modules": {
                "required": True,
                "default": None,
                "options": list(MODULES_MAP.keys()),
            },
        },
    },
    "fundamentals": {
        "path": "https://query2.finance.yahoo.com/ws/fundamentals-timeseries/v1/finance/timeseries/{symbol}",
        "response_field": "timeseries",
        "request": {
            "period1": {"required": True, "default": 493590046},
            "period2": {"required": True, "default": int(time.time())},
            "type": {
                "required": True,
                "default": None,
                "options": FUNDAMENTALS_MAP,
            },
            "merge": {"required": False, "default": False},
            "padTimeSeries": {"required": False, "default": False},
        },
    },
}

USER_AGENTS = [
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 "
    "Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 '
    'Safari/537.36'
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.129 "
    "Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 "
    "Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
]
